/******************************************************************
Section __: Reduce the number of obs in the data set: the slide is crowded
*******************************************************************/
data MyClass;
set SAShelp.class;
if mod(_N_,7) in (0,1,2,4,6,7) ;
run;

/******************************************************************
Section __: titles and summary statistics
*******************************************************************/
proc SQL _method _tree; 
select  left(put(sum(sex='F'),2.0) ) as FemCount
	   ,left(put(sum(sex='M'),2.0) ) as MaleCount
       ,avg(height) as AllAvgHeight
       ,avg(age)   as AllAvgAge
       ,sum(height*(sex='F'))/sum(sex='F') as FemAvgH
	   ,sum(height*(sex='M'))/sum(sex='M') as MaleAvgH  
       ,sum(Age*(sex='F'))/sum(sex='F')    as FemAvgAge
	   ,sum(Age*(sex='M'))/sum(sex='M')    as MaleAvgAge  
into   :FemCount             ,:MaleCount 
       ,:AllAvgHeight        ,:AllAvgAge 
       ,:FemAvgH             ,:MaleAvgH  
       ,:FemAvgAge           ,:MaleAvgAge
  from MyClass 
;
%put _user_;
title1 'AllAvgAge=&AllAvgAge  and  AllAvgHeight=&AllAvgHeight';
title2 'FemAvgAge=&FemAvgAge  and MaleAvgAge=&MaleAvgAge';
title3 'FemAvgH=&FemAvgH     and MaleAvgH=&MaleAvgH';
title4 'FemCount=&FemCount   and   MaleCount=&MaleCount';


%put _user_;
title1 'AllAvgAge=&AllAvgAge &  FemAvgAge=&FemAvgAge & MaleAvgAge=&MaleAvgAge' ; 
title2 ' AllAvgHeight=&AllAvgHeight & FemAvgH=&FemAvgH   & MaleAvgH=&MaleAvgH';
title3 'FemCount=&FemCount   and   MaleCount=&MaleCount';

/******************************************************************
Section __: examples
*******************************************************************/
Proc SQl; select Name, sex, count(*), age, avg(age), age*12 as AgeMo
 from MyClass
   where sex='M' and calculated AgeMo gt 135
   group by sex
 having substr(name,1,1) NE 'J'  
   and age/avg(age) GT 1.043;

Proc SQL;
Select Name,   sex,   avg(height)
 From MYClass
 Where age GT 11
 Group by Sex
 Having Substr(name,1,1)='J' 
	and height GT avg(height);

Proc SQL;
 Select avg(height) as  AvgHt
 From MYClass;

 Proc SQL;
 Select avg(height) as  AvgHt
 From MYClass
 Group by Sex;


 Proc SQL;
Select Name,   sex,   avg(height)
 From MYClass
 Where age GT 11
 Group by Sex
 Having Substr(name,1,1)='J' 
	and height GT avg(height);

options nocenter ps=30;
proc SQL _method _tree ;
footnote 'all detail obs pass through detail path';
select name , sex, age
from MyClass as C;
run; 
footnote '';



proc SQL _method _tree;
footnote1 'two lines, and two variables, go through the grouping path.'; 
footnote2 'sex is dropped late in the process and not not printed';
select  avg(age) as AvAge 
from MyClass as C
group by sex;
run; 


Proc SQL _method _tree;
footnote1 'Lists all sex values, ordered by sex';
footnote2 'WARNING: A GROUP BY clause has been transformed into an ORDER BY clause ';
footnote3 'because neither the SELECT clause nor the optional HAVING clause of associated ';
footnote4 'table-expression referenced a summary function. ';
select  sex  
from MyClass as C
group by sex;


Proc SQL _method _tree;
footnote1 'Two obs through the grouping    path, none through detail path. ';
footnote2 'Support for idea that all data    goes to groping path is that there is ';
footnote3 'No note in the log that mentions    a re-merging w/data afterwards';
select  sex, avg(age) as AvgAge  
from MyClass as C
group by sex;
run; 



proc SQL _method _tree;
select  
from MyClass as C
having sex='F';

proc SQL _method _tree;
select  
from MyClass as C
group by sex
having sex='M';

******???????????????????******************;
Proc SQL _method _tree;
footnote1 'detail lines, plus grouping var   (to allow merging), go through detail    path';
footnote2 'two obs (and two vars- sex  & AvgAge) go through grouping path.  ';
footnote3 'After detail and grouping paths merged, the grouping var is   dropped';

select  name , avg(age) 
from MyClass as C
group by sex;

Proc SQL _method _tree;
footnote1 'Lists all Name values and sex, ordered   by sex  name & sex sent through detial path,   file is sorted ';
footnote2 'WARNING: A GROUP BY clause has   been transformed into an ORDER BY clause     because neither the SELECT clause nor the   optional HAVING clause of the associated  table-  expression referenced a summary function. ';

select  name , sex 
from MyClass as C
group by sex;



Proc SQL;
Footnote1 'The avg in the having sends all filtered obs to the summary path';
Footnote2 'Look at number of obs';

select name, count(*) as obs, age 
from MyClass  
having age GT avg(age);
run;

Proc SQL;
Footnote1 'The avg in the having sends all filtered obs to the summary path';
Footnote2 'Look at number of obs';

select name, count(*) as obs, age 
from MyClass 
group by sex 
having age GT avg(age);
run;


