/******************************************************************
Section __: Reduce the number of obs in the data set: the slide is crowded
*******************************************************************/
data MyClass;
set SAShelp.class;
if mod(_N_,7) in (0,1,2,4,6,7) ;
label Name="Student name";
run;

/******************************************************************
Section __: Comparisons : Rules For SQL
*******************************************************************/
/*
<NOT> BETWEEN condition	.	
<NOT> CONTAINS condition	
<NOT> EXISTS condition		
<NOT> IN condition	
IS <NOT> condition	See IS condition.	
<NOT> LIKE condition	See LIKE condition.	
=, eq	equals	
=, ^=, < >, ne	does not equal	
>, gt	is greater than	
<, lt	is less than	
>=, ge	is greater than or equal to	
<=, le	is less than or equal to	
=*	sounds like (use with character operands only).
eqt	equal to truncated strings (use with character operands only).
gtt	greater than truncated strings	
ltt	less than truncated strings	
get	greater than or equal to truncated strings	
let	less than or equal to truncated strings	
net	not equal to truncated strings	
&, AND	indicates logical AND	
|, OR	indicates logical OR	
, ^, NOT	indicates logical NOT	


BETWEEN condition sql-expression <NOT> BETWEEN sql-expression AND sql-expression  
	Selects rows where column values are within a range of values. 
	The sql-expressions must be of compatible data types. They must be either all numeric or all character types.
	Because a BETWEEN condition evaluates the boundary values as a range, it is not necessary to specify the smaller quantity first.
	You can use the NOT logical operator to exclude a range of numbers, for example, to eliminate customer numbers between 1 and 15 (inclusive) so that you can retrieve data on more recently acquired customers.
	PROC SQL supports the same comparison operators that the DATA step supports. For example: 
	   x between 1 and 3	x between 3 and 1	   1<=x<=3	   x>=1 and x<=3

CONTAINS condition  sql-expression <NOT> CONTAINS sql-expression 
	Tests whether a string is part of a column's value. 

EXISTS condition   <NOT> EXISTS (query-expression)  
	Tests if a subquery returns one or more rows.
	The EXISTS condition is an operator whose right operand is a subquery. 
		The result of an EXISTS condition is true if the subquery resolves to at least one row.
		The result of a NOT EXISTS condition is true if the subquery evaluates to zero rows. 

IN condition sql-expression <NOT> IN (query-expression | constant <, ... constant>) 
	Tests set membership. 
	An IN condition tests if the column value that is returned by the sql-expression on the left 
		is a member of the set (of constants or values returned by the query-expression) on the right. 
	The IN condition is true if the value of the left-hand operand 
		is in the set of values that are defined by the right-hand operand.

LIKE condition sql-expression <NOT> LIKE sql-expression 
	Tests for a matching pattern.
	The LIKE condition selects rows by comparing character strings with a pattern-matching specification. 
		It resolves to true and displays the matched string(s) if the left operand matches the pattern specified by the right operand.
	The ESCAPE clause is used to search for literal instances of 
		the percent (%) and underscore (_) characters, which are usually used for pattern matching.

	Patterns are composed of three classes of characters: 
		underscore (_)		--> matches any single character.
		percent sign (%) 	--> matches any sequence of zero or more characters.
		any other character --> matches that character.
			Note:   When you are using the % character, be aware of the effect of trailing blanks. 
			You may have to use the TRIM function to remove trailing blanks in order to match values.  

		These patterns can appear before, after, or on both sides of characters that you want to match. 
		The LIKE condition is case-sensitive. 
		The following list uses these values: Smith , Smooth , Smothers , Smart , and Smuggle . 
		'Sm%'  		-->	matches Smith , Smooth , Smothers , Smart , Smuggle .
		'%th'  		-->	matches Smith , Smooth .
		'S__gg%'	--> matches Smuggle .
		'S_o' 		-->	matches a three-letter word, so it has no matches here.
		'S_o%' 		--> matches Smooth , Smothers .
		'S%th' 		-->	matches Smith , Smooth .
		'Z' 		-->	matches the single, uppercase character Z only, so it has no matches here.
		--------------------------------------------------------------------------------
		zearching for Literal % and _ 
		Because the % and _ characters have special meaning in the context of the LIKE condition, 
			you must use the ESCAPE clause to search for these character literals in the input character string.
		These example use the values app , a_% , a__ , bbaa1 , and ba_1 .
		The condition like 'a_%' matches app , a_% , and a__ , 
				because the underscore (_) in the search pattern matches any single character (including the underscore), 
				and the percent (%) in the search pattern matches zero or more characters, including '%' and '_'.
		The condition like 'a_^%' escape '^' matches only a_% , 
				because the escape character (^) specifies that the pattern search for a literal '%'.
		The condition like 'a_%' escape '_' matches none of the values, 
				because the escape character (_) specifies that the pattern search for an 'a' followed by a literal '%', 
				which does not apply to any of these values.
		--------------------------------------------------------------------------------
 		Searching for Mixed-Case Strings 
		To search for mixed-case strings, use the UPCASE function to make all the names uppercase 
				before entering the LIKE condition: 
				upcase(name) like 'SM%';


Sounds like Operator
Retrieving Values with the SOUNDS-LIKE Operator
  The SOUNDS-LIKE operator is based on the SOUNDEX algorithm for identifying words that sound alike. 
  The SOUNDEX algorithm is English-biased and is less useful for languages other than English.
The SOUNDEX function encodes a character string according to an algorithm that was originally developed 
   by Margaret K. Odell and Robert C. Russel (US Patents 1261167 (1918) and 1435663 (1922)). 
The algorithm is described in Knuth, The Art of Computer Programming, 
The SOUNDEX function returns a copy of the argument that is encoded by using the following steps: 

Retain the first letter in the argument and discard the following letters:
	A E H I O U W Y
Assign the following numbers to these classes of letters: 
	 1: B F P V
 	 2: C G J K Q S X Z
 	 3: D T
 	 4: L
 	 5: M N
 	 6: R

If two or more adjacent letters have the same classification from Step 2, 
		then discard all but the first. (Adjacent refers to the position in the word prior to discarding letters.)
Examples:
|---------------CODE THESE STATEMENTS------------------|      RESULT IS:
x=soundex('Paul');							put x;				-->P4
word='amnesty';			x=soundex(word);	Put x;				-->A523
*/

/******************************************************************
Section __: Missing Values
*******************************************************************/
/*
Missing Character is a blank or a string of blanks
Missing numeric is (ususally) a period
Missing valuesa are equal to themselves and are: 1) first in sorts and 2) smaller than other vlaues

Consider missing values in:
Is Null or Is Missing
Is Like
Exists
Functions (count, sum, avg, min, Max etc. 
*/




/******************************************************************
Section __: SAS  code for Comparisons 
*******************************************************************/
*<NOT> BETWEEN condition;
proc SQL;
select name , sex, age
from MyClass
where age Between 13 and 14
order by age;
quit; 

proc SQL;
select name , sex, age
from MyClass
where age NOT Between 13 and 14
order by age;
quit; 

*character;
proc SQL;
select name , sex, age
from MyClass
where name Between "Al" and "James"
order by Name;
quit; 


proc SQL;
select name , sex, age
from MyClass
where name Between "Zeke" and "Jonah" /*Order is reversed*/
order by Name;
quit; 


Proc SQL;
create  table names /*NO as*/
(name char(7) );
insert into names
values ("art")
values ("Amie")
values ("Zeke")
values ("zander")
values ("1223")
values ("  ")
values ("       ")
;
quit;
proc SQL number;
select "*"||name||"*" from names order by name;
quit;

proc SQL;
select name , sex, age
from MyClass
where name Between "Ambrose" and "aaron" 
order by Name;
quit; 


*<NOT> CONTAINS  Tests whether a string is part of a column's value. ;
Proc SQL;
select name, sex, age
from MyClass
where name contains "L";
run;

Proc SQL;
select name, sex, age
from MyClass
where upcase(name) contains "L";
run;

Proc SQL;
select name, sex, age
from MyClass
where upcase(name) contains "FF" or upcase(name) contains "CE";
run;

Proc SQL number _method _tree;
title "produces very odd results";
select name, sex, age
from MyClass
where upcase(name) contains "FF" or "CE";
run;


*<NOT> EXISTS condition;
*	Tests if a subquery returns one or more rows.
	The EXISTS condition is an operator whose right operand is a subquery. 
		The result of an EXISTS condition is true if the subquery resolves to at least one row.
		The result of a NOT EXISTS condition is true if the subquery evaluates to zero rows. ;
proc SQL ;
select age, sex, name  from MyClass as Outer
	where exists (select Name from MyClass as Inner
					where Outer.age EQ Inner.age and 
						  Outer.Sex NE Inner.Sex)
Order by Outer.age, Outer.sex;

proc SQL ;
select age, sex, name  from MyClass as Outer
	where NOT exists (select Name from MyClass as Inner
					where Outer.age EQ Inner.age and 
						  Outer.Sex NE Inner.Sex)
Order by Outer.age, Outer.sex;

*<NOT> IN condition  Tests set membership. 
IN condition sql-expression <NOT> IN (query-expression | constant <, ... constant>) 
	An IN condition tests if the column value that is returned by the sql-expression on the left 
		is a member of the set (of constants or values returned by the query-expression) on the right. 
	The IN condition is true if the value of the left-hand operand is in the set of values that are defined by the right-hand operand.
    *IS <NOT> condition	See IS condition.	;
proc SQL ;
select age, sex, name  from MyClass
	where SEX IN  ("F","U");

proc SQL ;
select sex, age, name  from MyClass as Outer
	where Name IN (Select Name from MyClass as Inner
					where outer.sex=Inner.sex
           			having(outer.age=max(inner.age) or outer.age=min(inner.age)
                         )   )
order by Sex , age;
	
proc SQL ;
select sex, age, name  from MyClass as Outer
	where Name, age IN (Select Name, age from MyClass as Inner
					where outer.sex=Inner.sex
           			having(outer.age=max(inner.age) or outer.age=min(inner.age)
                         )   )
order by Sex , age;

*<NOT> LIKE condition	Tests for a matching pattern.
LIKE condition sql-expression <NOT> LIKE sql-expression 
	The LIKE condition selects rows by comparing character strings with a pattern-matching specification. 
		It resolves to true and displays the matched string(s) if the left operand matches the pattern specified by the right operand.
	The ESCAPE clause is used to search for literal instances of 
		the percent (%) and underscore (_) characters, which are usually used for pattern matching.

	Patterns are composed of three classes of characters: 
		underscore (_)		--> matches any single character.
		percent sign (%) 	--> matches any sequence of zero or more characters.
		any other character --> matches that character.;

proc SQL;
select name from MyClass
where name Like "_a"; 

proc SQL;
select name from MyClass
where name Like "_a%"; 

proc SQL;
select name from MyClass
where name Like "%ff"; 

proc SQL;
select name from MyClass
where name Like '%ff';

proc SQL;
select name from MyClass
where name Like '%ff%';


*=, eq	equals;
*=, ^=, < >, ne	does not equal;
*>, gt	is greater than;
*<, lt	is less than;
*>=, ge	is greater than or equal to;
*<=, le	is less than or equal to;
*=*	sounds like (use with character operands only);
*eqt	equal to truncated strings (use with character operands only);
*gtt	greater than truncated strings;
*ltt	less than truncated strings;
*get	greater than or equal to truncated strings;
*let	less than or equal to truncated strings;
*net	not equal to truncated strings;
*&, AND	indicates logical AND;
*|, OR	indicates logical OR;
*, ^, NOT	indicates logical NOT;



/******************************************************************
Section __:
*******************************************************************/ 
/*Null Values 
According to the ANSI Standard for SQL, a missing value is called a null value. 
It is not the same as a blank or zero value. 
However, to be compatible with the rest of SAS, PROC SQL treats missing values the same as blanks or zero values, 
and considers all three to be null values. This important concept comes up in several places in this document.
*/
Proc SQL;
create  table names_N_Num /*NO as*/
(name char(7)
,NumVal num);
insert into names_N_Num
values ("art",22)
values ("Amie",.)
values ("Zeke",11)
values ("zander",55)
values ("1223",123)
values ("  ",88)
values ("       ",.);
quit;

options nocenter;
proc SQL;
select * 
from names_N_Num
where Name is missing;
quit;

options nocenter;
proc SQL;
select * 
from names_N_Num
where Name is Null;
quit;


options nocenter;
proc SQL;
select * 
from names_N_Num
where NumVal is missing;
quit;

options nocenter;
proc SQL;
select * 
from names_N_Num
where NumVal is Null;
quit;

Proc SQL;
select * 
from names_N_Num
where name is " ";
quit;


Proc SQL;
select * 
from names_N_Num
where name = " ";
quit;

Proc SQL;
select * 
from names_N_Num
where name = "            ";
quit;
