/*******************************************************
Program:       Macro Examples: Examples from talk level 1 and level 2
By:                  rml

Purpose:        Contains code from semianr examples, class examples AND
                Two different levels of sas macro examples
                To provide sample code for people learning macros



Infiles:

Outfiles:   xxxxxxxxxxxxxxxx
Started:      XXXXXXXXXXXXXXXXXX
Finished:   XXXXXXXXXXXXXXXXXX

Usage:      Run this code and interpret it using the Map of SAS
            use this as a source for solutions in your programming.
                Often we just need to have a worked sxample to modifiy
            Read the comments.  Hopefully  they will help;

Mod. Info:  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

*************************************************************************/



/**************************************************************************

   Many of the examples should be run in sections look for *run to here;

**************************************************************************/




/***************************************************************************/
/***************************************************************************/
/**************   Code from talk   **********************************/
/***************************************************************************/
/***************************************************************************/

/****************************************************************************
 SECTION CFT_1:  Global macro variables
*****************************************************************************/

Data year;
infile datalines missover firstobs=2;
input @1 Month $char5.
      @ 10 Sales @15 Returns @20 inventory;
Datalines;
12345678901234567890
jan      60   12   15
feb      61    6   14
mar      59    6   13
apr      66    5   14
may      59    4   15
june     60    3   11
july     61    5   17
aug      63    4   16
sept     62    6   15
oct      64    6   13
nov      66    6   15
dec      67    3   17
jan      60   12   13
feb      66    7   15
mar      58    5   13
apr      65    7   13
may      60    5   14
june     63    7   12
july     63    4   16
aug      61    4   15
sept     61    5   13
oct      62    5   12
nov      65    3   14
dec      64    3   15

;
run;
PROC PRINT;
title "Just look at the data";
RUN;

*run to here;


%let month=jan;  /*using Global macros is poor programming practice */

***start of reports***;
proc means data=year;
title "report for &Month;  This is how people ususally start using macros";
title2 "A simple substitution of a value in a title and a where";
where month="&month";
run;

Proc print data=year;
where month="&month";
run;
title "";


*run to here;

/**********************************************
  SECTION CFT_2 RULES FOR MACRO ASSIGNMENT
***********************************************/
/*I want to stre proc print; run; in a macro and save my silf some typing */
/*This is a classic miswtake - the first semicolon ends the %let
 - must use macro quoting.  Macro quoting is covered in a different lecture)*/

%LET OOPS=Proc Print; run;  /*This attempts to assign proc Print; run; to a macro variable*/

%put _user_; /*This prints the user created macro variables to the sas log */

*run to here;

   /*This is the fix*/
%LET smile=%str(Proc Print; run; );
%put _user_;
%put &smile;
   /*Use the fix to print the last data set You used*/
&smile;

*run to here;


/***********************************************
  SECTION CFT_3 %let and the double ampersand
************************************************/
/*This is just proctice for the && and && and &&&... */
%let StNames =holding1;
%let holding1=holding2;
%let holding2=NY,NJ,DE;

*run to here;

 **run the lines below and see what you get;

%put &StNames;
*run to here;

%put &&StNames;
*run to here;

%put &&&StNames;
*run to here;

%put &&&&StNames;
*run to here;

%put &&&&StNames;
*run to here;

%put &&&&&&&StNames;
*run to here;


/*******************************************************
   SECTION CFT_4 the first example of a macro program
*******************************************************/
/*This allows us to discuss the %if %then %do %end */
data sales;
infile datalines missover;
input state $  zip  $ sales Prod $ ;
datalines;
PA      19103   20 Gizmo
PA      19104   30 DoDad
PA      19104   20 DoDad
PA      19104   10 Gizmo
PA      19103   20 Gizmo
PA      19104   30 DoDad
PA      19104   20 DoDad
PA      19104   10 Gizmo
DE      18939   13 DoDad
DE      18922   12 Gizmo
DE      18877   17 Gizmo
DE      18877   7  Gizmo
DE      18877   27 DoDad
NJ      08002   94 Scone
;
run;


%put _automatic_; /*put the Macro vars THAT SAS AUTOMATICALLY CREATES to the log */
*run to here;


%Macro Pprint; /*This uses global macro variables :-( */
title "Stuff for &State";
%if &sysdate=22JUL06 %then %do;
	 proc freq data=sales;
	 tables state/missing;  run;
%end;
	proc print data=sales;
	 where state="&state";
	 sum sales;
	 run;
%Mend Pprint;

%Macro Mmean;  /*This uses global macro variables :-( */
 proc means data= sales;
 where state="&state";
 run;
%Mend Mmean;
*run to here;


%let state=NJ; /*The use of global macro varibales is NOT encouraged!! See below*/
%Mmean;
*run to here;

%let state=DE; /*The use of global macro varibales is NOT encouraged!! See below*/
%Pprint;
*run to here;

%let state=PA; /*The use of global macro varibales is NOT encouraged!! See below*/
%Mmean;
*run to here;


/*Below is a better way to do the above - use local named macro variables*/


%Macro Pprint2(State=); /*This says"  "when I call the macro I will tell it the value of state"*/
title "Stuff for &State";
%if &sysdate=Friday %then %do;
 proc freq data=sales;
 tables state/missing;  run;
%end;
proc print data=sales;
 where state="&state";
 sum sales;
 run;
%Mend Pprint2;

%Macro Mmean2(state=);
 proc means data= sales;
 where state="&state";
 run;
%Mend Mmean2;
*run to here;

%Mmean2(state=NJ)
*run to here;


%PPrint2(state=DE) ;
*run to here;



/*****************************************************************
SECTION CFT_6 The classic FAILURE to interact with the data - A TIMING ERROR
******************************************************************/
/*We want to put a FLAG in the title, depending on the sum of the sales*/
/*We will tell manages, in the title of the report, if the sales were high or low*/


proc summary  data=sashelp.shoes;/*We need the sum of sales in a data set*/
output out= for_macro
  (where=(_type_=0))   sum(sales)=SmRv;  run; /*SmRv stands for Sum of the revenue*/

proc print data=for_macro;  /*Just look at the data set*/
run;
*run to here;


data _null_;  /*use the Sum Revenue, in an If, to decide what walue the macro variable Tval (Title Variable) should have*/
set for_macro;
if SmRv GE 0  and
    SmRv LE 9999 then  %let Tvar=LOW month; ;
else if SmRv GT 9999 then
  %let Tvar=HIGH month; ;
else  %let Tvar=missing; ;
run;

%put _user_;
*run to here;

proc print data=sashelp.shoes(obs=5);  /*%lets evaluate at the top of the input stack*/
title "How was this month?  It was a &Tvar Month";
title2 "Tvar always has the value of missing";
run;
title "";
*run to here;

/********************************************************************
Parameter rules for Symput Symget and call execute
*********************************************************************/
/*
if quoted, that is the value, if not quoted, go to the PDV to get the value to use
Call Symput has two parameters (1) the macro variable name and (2) its value
Symget and call execute only have one parameter

*/


/********************************************************************
 SECTION CFT_7     A bit out of order, but seeing symget now will allow us to put
  the example that uses symput and symget in one section
*********************************************************************/
/*a very simple example of how Symget works, no business meaning*/
%let a1=the first;
%let a2=the middle;
%let a3=the end;
%put _user_;
*run to here;

data new;
infile datalines;
input FRMPDV $ @@;
datalines;
A1 A2 A3  A2
run;

data new;
length dest $12;
set new ;
dest=symget(FRMPDV);  /*Unquoted parameter look a the PDV for the value*/
run;

proc print data=new;
run;
*run to here;


/********************************************************************
SECTION CFT_8  having your program interact with the data
  Symput and symget in one example
*********************************************************************/

/*We need a business reason for doing this
Management wants to know, for each salesperson,
   their sales as a percent of their department sales
   and sales as a percentage of the total store sales.*/

data Sales;  /*the raw sales data*/
   input dept $ name $ sales;
   cards;
bedding Watlee 18000
kitchen Cannon 15000
tv Jones 9000
carpet Keller 20000
bedding Ives 16000
bedding Parker 9000
bedding George 8000
bedding Joiner 8000
carpet Ray 12000
carpet Jones 9000
kitchen White 8000
kitchen Banks 14000
kitchen Marks 9000
tv Smith 8000
tv Rogers 15000
tv Morse 16000
;
run;
proc print data=sales;
run;
*run to here;

proc means data=sales;
/*this data set will have total store sales on line 1 and department sales on following lines*/
/*Thesea re the denominators for the calculation of percentages*/
   class dept;
   var sales;
   output out=stats sum(sales)=s_sal;
run;

proc print data=stats;
   var dept s_sal;
   title "Summary of Sales Information";
   title2 "For Dusty Department Store";
run;
*run to here;

*******Create the macro variables with call symput;
data _null_;
   set stats;
/*Put the total sales and department sales into meaningfully named macro variables*/
if _n_=1 then
call symput('s_tot',s_sal); /*Sales total*/

else call symput('s'||dept,s_sal); /*Sales for bedding, tv etc */
run;

%put _user_;
*run to here;

*******use the macro variables with symget;
data new;
set sales;
pct_d= (sales / symget('s'||dept)) *100;

pct_t= (sales  /  &s_tot)  *100;

run;

Proc print data=new;
title "this is a cool example showing a mini-projct that uses call symput and symget";
run;
title "";
*run to here;


/********************************************
SECTION CFT_9 Call execute puts stuff on the input stack;
********************************************/
/*This is a very simple example showing what Call Execute does*/
/*Call execute put stuff on the input stack*/
/*The opower of call execue is in WHAT you have it put on the input stack*/

data _null_;

call execute
  ("Proc Print     data=sashelp.Class;");

call execute
  ('title "New Title";');

call execute   ("run;");
run;
*run to here;


/********************************************
SECTION CFT_10 Call execute - A more prActical example
********************************************/
/*We will use call execute to automate a job*/
/*The invironmant is that we have a large data file and several standard reports*/
/*We typically run a report on a subsets of the data (we slice the data several ways)*/

Data interest; /*I am interested in these report types on these states*/
infile datalines;
input state $ RType $;
datalines;
PA Sales
NJ Com
;
Run;
proc print data=interest;
title "this is my control file - it controls what progamas get run and how they subeset the data";
run;
title "";
*run to here;

Data NE_regn; /*the BIG :-)  data file */
infile datalines missover;
input    state $ Person $ Sales Com;
datalines;
NJ   Shira    100   10
CT   Vijay    700   73
DE   Ellen    600   62
PA   Ed       300   34
NJ   Sarah    400   44
CT   Val      600   56
DE   Eve      400   39
PA   Eric     400   42

;
run;
Proc print data=NE_regn;
title "/*the BIG :-)  data file */ ";
run;
*run to here;

%macro SSales(state=); /*One of our standard reports*/
proc print     data=NE_regn;
title "Sales for &state";
title2 "slightly different from slides";
title3 "Use of named parameters is Better coding practice";
var person   Sales;
where state ="&state";
run;
%Mend SSales;

%macro CCom(state=);   /*One of our standard reports*/
proc print     data=NE_regn;
title "Comm. for &state";
title2 "slightly different from slides";
title3 "Use of named parameters is Better coding practice";
var person   com;
where state ="&state";
 run;
%Mend CComm;
*run to here;

*Call execute puts stuff on the input stack;
/*Use call execute to automate our jobs*/
data _null_;
set interest;
if upcase(Rtype)="SALES"    then   call execute('%SSales(state='||state||')');
else if upcase(Rtype)="COM" then   call execute('%CCom(state='||state||')');
run;
*run to here;

/***************************************************************************/
/***************************************************************************/
/**************   IN CLASS EXERCISES         **********************************/
/***************************************************************************/
/***************************************************************************/


/**************************************************************************
Section IN_CE_1  QUOTES AFFECT RESOLUTION OF THE &
**************************************************************************/
Proc Print DATA=SASHELP.SHOES; /*SLIGHLTY DIFFERENT FROM CLASS*/
title "Sales Listing using automatic variables in the footnote";
footnote  'Acme Co. Run on: &Sysday &Sysdate';
var REGION SALES RETURNS;
Run;
*run to here;

/**************************************************************************
Section IN_CE_2 PUTTING THE RUN TIME IN YOUR FOOTNOTE OR TITLE
    QUOTES AFFECT RESOLUTION OF THE &
**************************************************************************/
**SUB-example 1 - single quotes block macro resolution;
Proc Print data=SASHELP.SHOES(OBS=5);
title "Sales Listing";
footnote Acme Co. Run on: &Sysday &Sysdate;
var REGION SALES RETURNS;
Run;
*run to here;


**SUB-example 2;
Proc Print data=SASHELP.SHOES(OBS=5);
title 'Sales Listing';
footnote "Acme Co. Run on: &Sysday, &Sysdate";
var REGION SALES RETURNS;
Run;
*run to here;


**SUB-example 3 - use sysfunc to get the REAL RUN TIME/DATE in a title or footnote;
  options mlogic mprint symbolgen;
Proc Print data=SASHELP.SHOES(OBS=5);
title1 "Sales Listing: run on %sysfunc(datetime(),datetime16.)";
TITLE2 "COMPARE TIME IN THE TITLE WITH THE TIME IN THE FOOTER";
footnote "Acme Company Run: &sysday &sysdate &SYSTIME";
footnote2 '&sysday &sysdate &SYSTIME are "loaded" when SAS is started';
var REGION SALES RETURNS;
Run;
*run to here;

/************************************************************
Section IN_CE_3  Evaluating ampersands
*************************************************************/
/*The section heading says it all*/
%let File11         =Y1999      ;
%let Mac21=     Like   ;
%let Mac31=I      ;
%let File12=Y2000      ;
%let Mac22=Outlet      ;
%let Mac32=Fun;
%let Lib13=Sales      ;
%let Mac23=here      ;
%let Mac33=snow;
%let Lib14=  Fish      ;
%let Mac24=it      ;
%let Mac34=at;
%let Mac15=Man      ;
%let Mac25=people      ;
%let Mac35=cold;
%let Mac16=Maine      ;
%let Mac26=any      ;
%let Mac36  =mac15;
*run to here;

%put  &mac31 &mac21 &mac24 &mac23;
*run to here;

%put &mac33&mac15;
*run to here;

%put Mail&mac15;
*run to here;

%put &mac15Eater;
*run to here;

%put &mac15.Eater;
*run to here;

%put &Lib14.er&Mac15;
*run to here;

%put &Mac15 &Mac26 &Lib13 &Mac34 the &Mac22;
*run to here;

/*using the . as an explicit "end of macro name" indicator*/
%put set &Lib13&File11;
*run to here;

%put set &Lib13.&File11;
*run to here;

%put set &Lib13..&File11;
*run to here;

/************************************************************
Section IN_CE_4  Indirect addressing
*************************************************************/

data indirect;
infile datalines;
input @1 region $char5. @10 Cust $char8.   @20 sale  6.2     ;
datalines;
regn1    Acme         133.33
regn2    A&P           233.33
regn3    TrJoe         333.33
regn4    Acme         433.33
regn2    A&P           533.33
regn4    Mom&Pop  633.33
regn1    Acme         733.33
regn3    Acme         833.33
regn4    Mom&Pop  933.33
;
run;
*run to here;

%macro indir(PointV=); /*note the use of named macro parameter :-) */
%let regn1=Sally Struthers;
%let regn2=Edith Bunker;
%let regn3=Rob Reiner;

PROC PRINT DATA=INDIRECT;
WHERE REGION="&PointV";
TITLE "SALES: &&&PointV the manager of &PointV";
Run;
/*for fun comment out the run - re-compile and re-run*/
%mend indir;
*run to here;


%indir(pointv=regn1);
*run to here;

%indir(pointv=regn3);
*run to here;


/************************************************************
Section IN_CE_5  looping and the macro reference environment
*************************************************************/
/*This is another example of automating a program (Making it data driven)*/

data one;
infile datalines missover firstobs=2;
input @1 Rep $char8.
         @11 Prod $char9.
         @22 Uni;

datalines;
123456789012345678901234567890
Maryanne  widget     12
Maryanne  gizmo      22
Maryanne  chatchkie  22
Ian       widget     12
Ian       gizmo      22
russ      widget      8
russ      gizmo       4
;
run;

Proc print data=one;
run;
title ""; footnote "";
*run to here;

Proc freq data=one;
tables rep /out=repsales(drop=percent);
run;

data _null_;
set repsales end=eof;
call symput("RepNo"||left(put(_n_,1.)), rep);
call symput("Utsby"||left(put(_n_,1.)),count);
if eof=1 then
  do;
    call symput("LoopLim",_n_);
  end;
run;
%put _user_;
*run to here;

/*Look at what we did- put macros to the log*/
%put &RepNo1 has sales of &UtsBy1;
%put &RepNo2 has sales of &UtsBy2;
%put &RepNo3 has sales of &UtsBy3;
*run to here;


%macro printing; /*use a loop to access/use each macro variable*/
%put Before loop starts  i=&i;                      /*this line gives a hint on internal processing*/

%do i = 1 %to &looplim;
  %put inside the loop  BEFORE the run i=&i;        /*this line gives a hint on internal processing*/
     proc print data=one;
       title "sales for &&repno&i -  not centered";
       where Rep="&&repno&i";
       sum uni;
       run;
       title "";
  %put inside the loop - after the run i=&i; run;   /*this line gives a hint on internal processing*/
%end;
 %put Outside the loop i=&i;                        /*this line gives a hint on internal processing*/

%mend printing;

%printing;
*run to here;


/************************************************************
Section IN_CE_6  checking for the existance of a file - sysfunc
                 this is an automation technique
*************************************************************/
/*if you are going to automate a job (like a payroll )
  it is good to check that  you have all your source files before you start the main program*/

%macro grace1(dsn=);
   %if %sysfunc(exist(&dsn)) %then
   %do;
      proc print data=&dsn;  /*this print is our "main program" - pretty small in this example*/
       run;
   %end;
%else  %put !!!!!!!!!!!!!  File &dsn NOT FOUND !!!!!!!!!!!!!!!!!!;
%mend grace1;


%grace1(dsn=one);
*run to here;

options nomlogic nomprint nosymbolgen;
%grace1(dsn=two); /* this does not exist*/
options mlogic mprint symbolgen;
*run to here;


/************************************************************
Section IN_CE_7  %eval - doing math with the macro processor
*************************************************************/
/*the macro processor is a text processing subroutine and does math poorly*/

%let test1 = (6 + 22);       /*this is not evaluested*/
%let test1A=%eval(6 + 22);  /*this is a way to do INTEGER MATH - you can also do floating decimal point math*/

%put test1 is not evaluated and equals   &test1    but test1A evaluates so Test1A equals &test1A;
run;
*run to here;

%let test1B=%eval(7 / 2);  /*this is a way to do INTEGER MATH - you can also do floating decimal point math*/
%put &test1B;

%let test1C=%SysEvalF(7 / 2);  /*this is a way to do INTEGER MATH - you can also do floating decimal point math*/
%put &test1C;


/************************************************************
Section IN_CE_8  OPTIONAL Macro processor confusion -
       is GE a company or a mnemonic for Greater than or Equal to?
       This is a macro fragment - It will not run for two reasons"  it is a fragment and it has quoting issues
       This is an intro to the problem of Macro quoting - No answer given in this talk
*************************************************************/
* We are checking to see if the company is Lucent Technologies -> LT;
/*This shows why people need to learn to macro quote (the quick fix ofen works as well)*/
/*the macro says GE NE LT is False!!!!*/

options mlogic symbolgen;
%Macro Stocks(Symb=);
/* %if  "&symb"  NE  "LT" %then*/ /*Using double quotes is often a quick fix*/

 %if  "&symb"  NE  "LT"   %then   /*This syntax confuses the macro processor*/
   %do;
    proc print data=AnnRpt;
    where stock="&symb";
    run;

  %end;
%mend stocks;

%stocks(Symb=GE);
*run to here;



/************************************************************
Section IN_CE_9  Where are macros stored ?
*************************************************************/
proc catalog Cat=work.sasmacr;
title "Where did my macro Go??";
title2 "it went to a catalog in work";
contents;
quit;
title "";
*run to here;


*************************************************************************/

/***************************************************************************/
/***************************************************************************/
/**************     LEVEL ONE Examples  of cool code ***********************/
/***************************************************************************/
/***************************************************************************/

/***********************************************************************
  SECTION : ** system options useful for macros
************************************************************************/
*Nocenter left aligns output in the listing window and makes it easier to read;
*Mprint Mlogic and Symbolgen cause SAS to write more "stuff" to the log.  Stuff about the macros you  run;
options nocenter;
options mprint mlogic symbolgen;  /*Causes additional info to be sent to the log*/
*run to here;


/***********************************************************************
  SECTION : ** print the datasets we will use in these examples;
************************************************************************/
/*All sas students shold know that these data sets exist*/
proc print data=sashelp.class;
title "This is the data set used by most examples";
run;
*run to here;

proc print data=sashelp.shoes;
title "this data set can be used by readers to create their own reports/macros";
run;
title "";
*run to here;


/***********************************************************************
  SECTION 1: Macros recall values for you
            Checking what is in the macro symbol table - What you have stored (and, therfore, what you can recall)
            %let creates a global macro variable and use og GLOBAL macro variables is to be avoided when you can
************************************************************************/
proc print data=sashelp.class;
run;
*run to here;

*Lets see how we can look at the contents of the macro table;
****;
%let MVsex="M";   /*Global Variable ;-(*/
/*Macro variables are text. People get confused about the need for quoting*/
* MVsex is not quoted, it contains quotes */
**** you can look at macro values in many ways;

%put  _user_; /*Shows all the user defined macro variables*/
%put we want the value of a variable =  &mvsex mvsex;  /*&macroVarName shows the vlaue of that variable */
*run to here;

/*if you put quotes IN the macro variable itself, you do not need hem when you use the macro*/
/*INtentional error below*/
proc print data=sashelp.class;
where sex="&MVsex";
/*the log looks odd.  The error is because of double quoting of M.*/
/*  we must fix the macro variable- or the where - */
run;
*run to here;


/**MPrint Mlogic and Symbolgen are options for checking -
 only Symbolgen "works" in this simple macro use - Mlogic and Mprint do not "kick in"*/
options mprint mlogic nosymbolgen /*nomprint nomlogic nosymbolgen  */;
proc print data=sashelp.class;
where sex=&MVsex;  /*we must fix the macro - or the where - here we fix the where- the quotes are IN MVsex*/
run;
*run to here;

%let MVsex=M;  /*No quotes in the macro variable*/
proc print data=sashelp.class;
where sex="&MVsex";  /**/
run;
*run to here;



**We need to know what isw in the macro symbol table- inquiring minds want to know *********************;
*Several different ways of looking into the Macro Symbol table are shown below;
%put _user_;
*run to here;

%put _Global_;
*run to here;

%put _Automatic_;
*run to here;

%put _All_;
*run to here;


%put _user_;
*run to here;

*or you can use SQL to see the Macro Symbol Table- very cool stuff, but it is a bit advanced;
proc sql;
describe table dictionary.macros;
select distinct scope from dictionary.macros
where scope="GLOBAL";
quit;
*That is it- above are all the ways I know how to check the symbol table;
*run to here;

proc print data=SASHELP.Vmacro;
where scope="GLOBAL";
quit;



/***********************************************************************
  SECTION 2a: macro example   MPRINT(ONESEX): lines in the log are the result of  MPRINT
          This example shows named macro parameters (these are local and are good programming practice)
************************************************************************/
/*explore case sensitivity. - This is really the playing with the normal SAS case sensitivity*/

/*Mprint Mlogic and symbolgen are useful macro debugging tools*/
/*They can produce too much stuff. turn them off with NOmprint NOMlogic and Nosymbolgen*/
options mprint mlogic mprint;


%macro OneSex(ListSex= );
proc print data=sashelp.class;
where sex="&ListSex";
run;

proc means data=sashelp.class;
where sex="&ListSex";
run;
%mend;
*run to here;


%OneSex(ListSex=M); /*this works*/
*run to here;

%OneSex(ListSex=m); /*this shows the case sensitivity - returns 0 Obs*/
*run to here;

%OneSex(ListSex=F); /*this works*/
*run to here;

%put _user_;
*run to here;

/***********************************************************************
  SECTION 2b: macro example:  A macro with two criteria in the where
************************************************************************/
/*This is just an expansion of the examples above - we pass two parameters to the macro*/
/*We know the filtering variables when we write the macro, but not the value of interest*/

%macro TwoCrit(ListSex=
              ,listAge= );
%put ***Macro variables printed from inside the macro - notice LOCAL variables***;
%put _user_;
proc print data=sashelp.class;
where sex="&ListSex" and age <=&ListAge;
run;

proc means data=sashelp.class;
where sex="&ListSex" and age=&ListAge;
run;

%mend TwoCrit;
*run to here;


%TwoCrit(ListSex=F
        ,listAge=15 );
%put _all_; /*Note that Listage and listsex have disappeared- they were local*/
run;
*run to here;



/***********************************************************************
  SECTION 2c: macro example: A macro where you can specify the where variable and its value
************************************************************************/
/*When we wrote this macro we did not know the filtering variable Or its value of interest */

%macro VarNVal(Wherervar=
              ,WhereValue= );
proc print data=sashelp.class;
where &Wherervar = &WhereValue ;
run;

proc means data=sashelp.class;
where &Wherervar = &WhereValue ;
run;
%mend VarNVal;
*run to here;

%VarNVal(Wherervar=Sex
        ,WhereValue="F" );
*run to here;

%VarNVal(Wherervar=Age
        ,WhereValue=15 );
*run to here;


/***********************************************************************
  SECTION 2D: macro examples:   Specify the where variable, its value and the relationsip
************************************************************************/
/*This is a little more complex The section heading says it all*/
/*Specify the where variable, its value and the relationsip when you run the macro*/

%macro VarRelVal(Wherervar=
              ,relat=%str(GE)  /*GE is a default - %str is macro quoting & too deep to explain here */
              ,WhereValue= );
proc print data=sashelp.class;
where &Wherervar &relat &WhereValue ;
run;

proc means data=sashelp.class;
where &Wherervar &relat &WhereValue ;
run;
%mend VarRelVal;
*run to here;

%VarRelVal(Wherervar=Age
  /*If we do not specify a value for relat, SAS uses the default*/
      ,WhereValue=14 );
*run to here;

%VarRelVal(Wherervar=Age
        ,relat=%str(=)
        ,WhereValue=15 );
*run to here;

/*The example below is a trick to "remove" a where caluse from a macro*/
%VarRelVal(Wherervar=1
/*A useful "Where" trick!!
    Where 1=1
  is always true and so prints all the obs */
        ,relat=%str(=)
        ,WhereValue=1 );
*run to here;

/***********************************************************************
  SECTION 3: indirect addressing and the &&
************************************************************************/
**An INtroduction to INDIRECT ADDRESSING AND THE &&;
%let month=january;
%let timep=&month;
%put month=&month timep=&timep;
*run to here;

%let StNames =holding1;
%let holding1=holding2;
%let holding2=NY,NJ,DE;
*run to here;

%PUT &StNames;
*run to here;

%PUT &&StNames;
*run to here;

%PUT &&&StNames;
*run to here;

%PUT &&&&&&&StNames;
*run to here;

%let ooops=%str(proc print; run;);  /*%str again - macro quoting  */
%put _user_;
*run to here;

&ooops;

/***********************************************************************
  SECTION 4:Rules for puting stuff in the Macro Symbol table
            The first semi-colon ends the command
            & and % are evaluated
************************************************************************/
** the first semocolon ends the command;
%let oops=proc print; run;
%put _user_;
*run to here;

%let oops2=%str(proc print; run;);
%put &oops2; /* note the note in the log about unmasking*/
*run to here;

%put _user_; /* note the boxes in the log if you have V8 (odd characters if you have V9)*/
/*Boxes are maked or quoted characters and can not be printed as they are stored*/
*run to here;

%put _all_;
*run to here;


**& and % (have to wait for a full understanding of %) are evaluated on macro assignment;
*put an & into the macro symbol table;
options mlogic symbolgen mprint;
%let time_period=&month;
%put &time_period &month;
*run to here;


****SECTION 4A-1***Kind advanced stuff.  a % is evaluated as it flows into the symbol table;
/*  ;-(  Note that the RESULTS of %macklist in the table, not the string %makelist*******  */
**shows a % being evaluated as it goes into the symbol table***********************;
/*We have three macro variables and we want to combine them into one, called ListOf By Variables*/


%let Max_of_bys=3;     /*count of "real" by macros defined below- Do not count By0 */
%let By0=%str(             );
  /* Must have B0 to get the overall answers - fill with spaces to length of longest By*/
%let By1=%str(By spec);
%let By2=%str(By cat);
%let By3=%str(By segment);
*run to here;


/*concatinate the values of the by variabels into one macro var- use this for sorting*/
%macro makelist;
  %do cntr=1 %to &Max_of_bys; %substr(&&by&cntr,4) %end;
%mend makelist;
*run to here;

/*below we try to put a % into the Macro Symbol Table*/
options mprint mlogic symbolgen;
%let LstOfByVars=%makelist;  /*%makelist RUNS as/before it is loaded into the macro symbol table*/
%put LstOfByVars=&LstOfByVars by1=&by1 by2=&by2 by3=&by3; /*where does the one space between the variables come from?*/
*run to here;

/***********************************************************************
  SECTION 5: an example of a %if
  Imagine that the data set is being refreshed as transactions occurr
  We want a daily report of sales and returns for a region
       and on Friday we want a report on inventory in all regions
         If we are short of inventory, we want to be able to get inventory from another region
************************************************************************/
/*This is a macro that uses a %IF to produce EXTRA analysis, if run on saturday*/


%macro mac_if(Subsidiary=);
                 /*Change the day of the week to change execution of the macro*/
%if %upcase(&sysday) EQ SATURDAY %then
  %do;
   title "End of Week Inventory report for &Sysday, &sysdate";
   proc summary data=sashelp.shoes missing;
    class  product region subsidiary  ;
    output out=invntry(drop= _Type_ _freq_) sum(sales inventory)= ;
    Types Product*Region*subsidiary;
    quit;

   Proc Print data=invntry uniform;
    by product ;
    run;
    title "";
  %end;

  Proc print data=sashelp.shoes;
   title "Listing of activity in subsidiary: &subsidiary";
   var region subsidiary product;
   where subsidiary="&Subsidiary";
   run;
   title "";


%mend Mac_if;
*run to here;

options mprint mlogic symbolgen;
%mac_if(Subsidiary=Ottawa);
*run to here;

/***********************************************************************
  SECTION 6: An Example of using creating and using macro variables -- See Call Symput (Below) and symget (section 7)
             this is sometimes called "creating a macro array"
************************************************************************/
/*This is the example from the seminar again*/
/*None of the individual commands make sense */
/*we need to see the whole project to undersand what goes on and, very importantly, WHY we did this this way*/

data Sales;
   input dept $ name $ sales;
   cards;
bedding  Watlee  18000
kitchen  Cannon  15000
tv       Jones   9000
carpet   Keller  20000
bedding  Ives    16000
bedding  Parker  9000
bedding  George  8000
;
run;

proc means data=sales;
 class dept;
   var sales;
   output out=stats sum=s_sal;
run;
*run to here;

proc print data=stats;
   var dept s_sal;
   title "Summary of Salary Information";
   title2 "For Dusty Department Store";
run;
*run to here;

data _null_;
   set stats;
if _n_=1 then
 call symput('s_tot',s_sal);
 else call symput('s'||dept,s_sal);
run;

/*above (creating a macro array  is NOT useful
  without seeing the technique for USING a macro array*/
/*Some code useing this array will be shown below Ex 11- think of the two examples together*/
%put _user_;
*run to here;


**This example is continued in section 7;


/***********************************************************************
  SECTION 7: Basics of Symget - a simple example of getting values from the macro table
                        Timing Issues
                        Using a data step (not the top of the Input stack) to get values
                                from the Macro Symbol Table
************************************************************************/
/*repeat of material - this was also included in the first section*/
/*Thik of this as a footnote- stuck in the middle of the example shown in 6 and 8*/
%let a1=the first;
%let a2=the middle;
%let a3=the end;
%put _user_;
*run to here;

data new;
infile datalines;
input FRMPDV $ @@;
datalines;
A1 A2 A3  A2
;
run;

data new;
length dest $12;
set new ;
dest=symget(FRMPDV);  /*Assign the value of the macro to the variable dest  */
run;

proc print data=new;
run;
*run to here;

**put on the debugger if you know how to use it;
*data new/debug;
*length dest $12;
*set new ;
*Executable="this line";
*dest=symget(FRMPDV);
*run;

/***********************************************************************
  SECTION 8: A more practical Example of Symget - use "Macro Array" created in Section 5

************************************************************************/
/*This is a seminar example - this is the end of the example started in 6*/
data new;
set sales;
pct_d= (sales /  symget('s'||dept)) *100;
stop1="here";

pct_t= (sales  /  &s_tot)  *100;
stop2="there";
run;

proc print data=new;
title "Section 7 - using the array created in Secion 5";
run;
*run to here;



/***********************************************************************
  SECTION 9: A simple call execute - it put stuff on the input stack

************************************************************************/
/*Also shown in section 1*/
*Call execute puts stuff on the input stack;
data _null_; /*this is pretty silly, but that is what execute does*/
call execute("Proc Print  data=SASHELP.class;");
call execute('title "New Title";');
call execute("run;");
run;
*run to here;


/***********************************************************************
  SECTION 10a: macros can be used to produce SMALL PIECES OF SAS statements

************************************************************************/
**this will not run-  I do not have data to support this;
/*You can look a tthe log to see what it generates*/
/*It generates, not a whole proc, but just a few tokens*/
/*I do not want to type
  Fiscal_year1997 Fiscal_year1998 Fiscal_year1999.....  Fiscal_year2006 and wrote a macro*/
/*people do this!! instead of
        Fiscal_year1997-Fiscal_year2006
               and it makes for tough reading*/

%macro years(start= , end=);
 %do i= &start %to &end;
   fiscal_year&i
 %end;
%mend years;


/* Sometimes you write a macro before the data arrives.
   You can check the log to see if the macro did what you wanted it to do
   EVEN THOUGH THE LACK OF A DATA SET CAUSES AN EXECUTION ERROR
   The log tells me that my macro is good*/

options nomlogic mprint nosymbolgen;
data combined;
set %years(start=1997, end=2006);
;
run;
*run to here;

/***********************************************************************
  SECTION 10b: macros can be used to produce SMALL PIECES OF  SAS statements

************************************************************************/
/*another example of using a macro to produce a small piece of code*/
/*makes for difficult reading*/

%macro MyWhere(start= , end=);
 %do i= &start %to %eval(&end-1);
   &i ,
 %end;
    &i
%mend MyWhere;

proc print data=sashelp.class;
where age in(%MyWhere(start=12,end=14));
run;
*run to here;

/***********************************************************************
  SECTION 10C: macros can be used to produce SMALL PIECES OF  SAS statements

************************************************************************/
/*another example of using a macro to produce a small piece of code*/
/*makes for difficult reading*/


%MACRO MyElse(ageLvl=
              ,sexLvl=
              ,class=      );
   else if age=&ageLvl and Sex="&sexLvl" then class="&class" ;

%mend MyElse;
*run to here;

options mprint;
data oops;
set  sashelp.class;

if age=10 and sex="M" then class="M Pre-teen";
%MyElse(ageLvl=10,SexLvl=F,class=F Pre-teen)
%MyElse(ageLvl=11,SexLvl=M,class=M Pre-teen)
%MyElse(ageLvl=11,SexLvl=F,class=F Pre-teen)
%MyElse(ageLvl=12,SexLvl=M,class=M Pre-teen)
%MyElse(ageLvl=12,SexLvl=F,class=F Pre-teen)
%MyElse(ageLvl=13,SexLvl=M,class=M Teen    )
%MyElse(ageLvl=13,SexLvl=F,class=F Teen    )
else class = "No Class";
run;

Proc Print data=oops;
run;
*run to here;

/***********************************************************************
  SECTION 10D: Capture, in a text file, A CLEAN COPY of what your code generates

************************************************************************/
/*Debugging macros can be a pain in the neck.
  You have to imagine what goes on or read a log that is often very cluttered
  You can capture the code that the macro generates with the trick below*/


%macro wrapper;
data oops;
set  sashelp.class;

if age=10 and sex="M" then class="M Pre-teen";
%MyElse(ageLvl=10,SexLvl=F,class=F Pre-teen)
%MyElse(ageLvl=11,SexLvl=M,class=M Pre-teen)
%MyElse(ageLvl=11,SexLvl=F,class=F Pre-teen)
%MyElse(ageLvl=12,SexLvl=M,class=M Pre-teen)
%MyElse(ageLvl=12,SexLvl=F,class=F Pre-teen)
%MyElse(ageLvl=13,SexLvl=M,class=M Teen    )
%MyElse(ageLvl=13,SexLvl=F,class=F Teen    )
else class = "No Class";
run;

Proc Print data=oops;
run;

%mend wrapper;
*run to here;



   /*the directory c:\TEMP MUST EXIST on your computer for this to run*/
   /*Check the log for a not about writing to an external file, then look a the file*/
options mprint mfile;
filename mprint "C:\temp\captured_code.txt";

%wrapper

options noMprint;
*run to here;


/***************************************************************************/
/***************************************************************************/
/**************     LEVEL TWO Examples   **********************************/
/***************************************************************************/
/***************************************************************************/


/***********************************************************************
  more examples below - See if these make sense

************************************************************************/


/***********************************************************************
  SECTION 11: Positional parameters ARE LOCAL and good programming practice
            Also, this illustrates a trick to "get rid of the where clause"
            This shows how global macros can interfere with other macros
************************************************************************/
%macro Positional_1(WhereCls=   /*This should be the whole where clause statement, without a ;*/
                   );
%put _user_;
proc print data=sashelp.class;
title "A local and global macro have the same name";
&WhereCls  ;
run;
%mend Positional_1;
*run to here;


%Positional_1(WhereCls=);
*run to here;


%Positional_1(wherecls=Where sex="M");
%put _user_;  /*there is no macro named WhereLcs when this statement executes*/
*run to here;


/***********************************************************************
  SECTION 12: Positional parameters ARE LOCAL and good programming practice
************************************************************************/

%macro Positional_2(WhereCls=);
%put "";
%put "Macro Evaluated when the macro is running";
%put _user_;
%put "";

proc print data=sashelp.class;
title "A local and global macro have the same name";
&WhereCls  ;
run;
%mend Positional_2;
*run to here;


%let wherecls=Some Darn Global Value from section 12;
%put _user_;
*run to here;


%Positional_2(WhereCls=Where sex="M");
*Note the two macro variables (GLOBAL and local to Positional_2) named wherecl;
%put _user_;
*run to here;


/***********************************************************
      SECTION 13: the OLD way of looping, using symput to create a macro array.
      We end up with lots of global macro variables in memory
      Symput is falling out of fashion and folks are using SQL

      This program prints a proc contnets and 10 obs for every SAS table in the library SASHELP
***********************************************************/
/*This macro produces a proc contents and 10 obs for every SAS table in the library SASHELP*/

data _null_;
set sashelp.vtable;
where memtype="DATA" and libname="SASHELP";
count+1;
call symput("dsn_"||left(put(count,3.0)),memname);
call symput("No_of_files",count);
         /*Gets called many times and the number just keeps getting bigger */
         /*we only care about the last time it is called*/
run;
%put _user_;
*run to here;

%macro oldway;
%do i=1 %to &No_of_files;
 title "contents and 10 obs from sashelp.&&dsn_&i;";
 proc contents data=sashelp.&&dsn_&i;
 run;
 proc print data=sashelp.&&dsn_&i(obs=10);
 run;
 title "";
%end;
%mend oldway;
*run to here;

%oldway;
*run to here;

/***********************************************************
      SECTION: 14 Semi-New Way of looping - SQL into ONE LOCAL macro variable
        and put upper limit of loop into &SQLOBS - use do loop
***********************************************************/
/*This macro produces a proc contents and 10 obs for every SAS table in the library SASHELP*/

%macro Chk_files;
options nocenter;
proc sql noprint;
select memname into :file_list separated by " "
      from dictionary.tables
     where memtype="DATA" and libname="SASHELP";
quit;
%let SQLOBS=&SQLOBS;
%put _user_;
*run to here;

%do i=1 %to &SQLOBS /*%while=(%scan(&file_list,&file_no) NE )) ;  */;
      %let thisfile=%scan(&file_list,&i);
   proc contents data=sashelp.&thisfile(obs=10);
   title "contents and 10 obs from sashelp.&thisfile";
   run;

    proc print data=sashelp.&thisfile(obs=10);
    run;
      title "";
%end; /*end of the loop*/
%mend Chk_files;
*run to here;

%Chk_files;
*run to here;


/***********************************************************
      SECTION: 15 Newest Way of looping.
                  Use SQL and scan the macro variable until scan returns a blank
***********************************************************/
/*This macro produces a proc contents and 10 obs for every SAS table in the library SASHELP*/
%macro Chk_files;
options nocenter;
proc sql noprint;
select memname into :file_list separated by " "
      from dictionary.tables
     where memtype="DATA" and libname="SASHELP";
quit;
%let SQLOBS=&SQLOBS;
%put _user_;

%let file_no=1;
%do %while(%scan(&file_list,&file_no) NE ) ;
      %let thisfile=%scan(&file_list,&file_no);
      proc print data=sashelp.&thisfile(obs=10);
      title "10 obs from sashelp.&thisfile";
    run;
   %let file_no=%eval(&file_no+1); /*endless loop if this is forgotten*/
%end; /*end of the loop*/
%mend Chk_files;
*run to here;

%Chk_files;
*run to here;


/***********************************************************
      SECTION 16: Just Playing around, Using the SQL and creating individual macro varaibles
***********************************************************/
/*the SQL "into" is used to create macro variables.  It is powerful and popular */
%macro Semioldway;
proc sql noprint;
select distinct memname into :infile1-:infile999
      from dictionary.tables
     where memtype="DATA" and libname="SASHELP";
quit;

%let Countoffiles=&SQLobs;
%Put _user_;

%do i=1 %to &Countoffiles;
 proc print data=sashelp.&&infile&i(obs=10);
 title "proc print data=sashelp.&&infile&i(obs=10);";
 run;
%end;
%mend Semioldway;
*run to here;

%Semioldway;
*run to here;

/***********************************************************
      SECTION 17 : Use call execute to automate a job
 WHEN WE GO TO A NEW DIRECTORY WE NEED TO LEARN WAHT IS IN THE DIRECTORY
  WE USUALLY PRINT THE CONTENTS OF THE FILES AND 10 OBSERVATOINS FROM EACH FILE
***********************************************************/
/*This is an example of using a contol file to automate a job*/
/*We want a contents and 10 obs for all the files in a directory*/

options nocenter;
proc sql noprint;
TITLE 'CREATE A CONTROL FILE - this file "Drives"/"passes parameters to" the macro';
create table file_list as
select memname
      from dictionary.tables
     where memtype="DATA" and libname="SASHELP";
quit;


%macro ContAnd10(DSN=); /*WRITE A MACRO TO BE CONTROLLED BY A CONTROL FILE */
proc contents data=sashelp.&dsn;
title "10 obs using: proc print data=sashelp.&dsn(obs=10);";
run;
proc print data=sashelp.&dsn(obs=10);
run;
%mend ContAnd10;
*run to here;

data _null_;
/*use the CONTROL file to call the macro and pass it parameters */
set file_list;
call execute('%ContAnd10(DSN='||memname||')');
*call executes calls the macro and passes it parameters;
run;
*run to here;



/***********************************************************
      SECTION 18 : Use call execute to automate a job
***********************************************************/


%macro printing;

%do i = 1 %to &looplim;

proc print data=one;
 title "sales for &&repno&i";
 where salesrep ="&&repno&i";
 sum units;
run;

%end;
%mend printing;
*run to here;




/***************************************************************
      SECTION 19A MORE JOB AUTOMATION - CREATE A MINI REPORT FOR EACH AGE in the file SASHELP.lcass
           Have SAS figure out how many ages there are in the data
           This is dynamic programming- the data tells us what to do
****************************************************************/
/*This is a data driven process - we ask the infile how many reports to generate*/

proc freq data=sashelp.class;
tables age/out=out_age(drop=count percent); /*We need a report for each level of age in the outfile*/
run;

proc print data=out_age;                   /*Look at the outfile*/
run;
*run to here;

data _null_;                               /*Create macro variables from the levels of age in out_age*/
set out_age end=eof;
call symput('age'||left(put(_N_,2.)),age);
if eof=1 then call symput("NumOfObs",_n_); /*This will be the upper limit of the looping*/
run;
%put _user_;
*run to here;

%macro age_loop;                           /*The looping macro*/
%do i= 1 %to &NumOfObs;
 Proc print data=sashelp.class;
 title "list of students of age=&&age&i";
 where age=&&age&i;
 run;
 title "";
%end;
%mend age_loop;
*run to here;

%age_loop;
*run to here;

/***************************************************************
      SECTION 19B MORE JOB AUTOMATION - CREATE A MINI REPORT FOR EACH SEX
           Have SAS figure out how many levels of sex there are in the data
           This is dynamic programming- the data tells us what to do
****************************************************************/
/*this is VERY similar to the example above, but 90% of learning is practice*/
proc freq data=sashelp.class;
tables sex/out=out_sex(drop=count percent);
run;

proc print data=out_sex;
run;
*run to here;

data _null_;
set out_sex end=eof;
call symput('Sex'||left(put(_N_,2.)),sex);
if eof=1 then call symput("NumOfObs",_n_);
run;
%put _user_;
*run to here;

%macro sex_loop;
%do i= 1 %to &NumOfObs;
 Proc print data=sashelp.class;
 title "list of students of sex=&&sex&i";
 where sex="&&sex&i";
 run;
 title "";
%end;
%mend sex_loop;
*run to here;

%sex_loop;
*run to here;

/***************************************************************
      SECTION 19C MORE JOB AUTOMATION - use SQL to CREATE A MINI REPORT FOR EACH SEX
           Have SAS figure out how many levels of sex there are in the data
           This is dynamic programming- the data tells us what to do
****************************************************************/
proc SQL;
select distinct sex into :SexList separated by " "
from sashelp.class;
quit;
%put &SexList;

%macro AutoMatic;
%let counter=1;
%do %while(%scan(&sexlist,&counter) NE );
  %let ThisSex=%scan(&sexlist,&counter);
  Proc print data=sashelp.class;
  title "listing of students in SASHELP.class whith sex=&ThisSex";
  where sex="&ThisSex";
  run;
  title "";

%let counter=%eval(&counter+1);

%end;
%mend AutoMatic;

%AutoMatic;

/***************************************************************
      SECTION 20 MORE JOB AUTOMATION -  TIMING of macro events
  When you run the macro
    Note the order of notes in the log
     - that can tell you the order of when things happened
  -
****************************************************************/
/*Note where the %puts are, in relation to the notes about run times*/
options source2 mprint;
data one;
infile datalines missover firstobs=2;
input @1 Rep $char8.          /*Sales Rep*/
         @11 Prod $char9.     /*Product Sold*/
         @22 Uni;             /*Units sold*/

datalines;
123456789012345678901234567890
Maryanne  widget     12
Maryanne  gizmo      22
Maryanne  chatchkie  22
Ian       widget     12
Ian       gizmo      22
russ      widget      8
russ      gizmo       4
;
run;


Proc freq data=one;
tables rep /out=repsales(drop=percent);
run;

proc print data=one;
run;
*run to here;


data _null_;
set repsales end=eof;
call symput("RepNo"||left(put(_n_,1.)), rep);
call symput("Utsby"||left(put(_n_,1.)),count);
if eof=1 then
  do;
    call symput("LoopLim",_n_);
  end;
run;

%put _user_;
*run to here;


%macro printing;
/*We will change the centering to nocenter and then change it back*/
/*This is mre than a courtesy - do it all the time*/
%local CentLvl;
data _null_;
 set sashelp.voption(where=(optname="CENTER"));
 call symput("CentLvl",setting);
run;
*run to here;


options nocenter;

 %put Outside the loop - Before loop starts  i=&i;
%do i = 1 %to &looplim;
 %put ; %put ; %put ;
 %put inside the loop BEFORE the run i=&i;
 proc print data=one;
 title "sales for &&repno&i -not centered";
  where Rep="&&repno&i";
  sum uni;
 run;
 title "";
 %put inside the loop - after the run i=&i;

run;
%end;

options &CentLvl; /*return system variable center? to original value*/
 %put Outside the loop - all looping/printing done about to shut down i=&i;
%mend printing;
*run to here;


%printing;
*run to here;


 proc print data=one;
 title "sales -print is outside the macro ";
 title2 "system option has been re-set and outp[ut IS NOT centered";
 run;
title "";
*run to here;

/*check if we reset the centering*/
proc options;
title "look for either CENTER or NOCENTER in the left hand column ";
run;
title "";
*run to here;

/***************************************************************

         THAT'S ALL FOLKS -- THE END

****************************************************************/
