/*
          THIS TOP SECTION IS A TAKE AWAY - A MEMORY JOGGER FOR AUTOMATION TRICKS
This top section (about 300 lines) is a quick recap of most of the useful automation tricks
If you need to steal code for a project, you might steal it from this top section
Below the big white gap (lines 388 to 425) , is the code that goes with the ppt slides
*/

/***********************************************************************
  SECTION : ** system options useful for macros
************************************************************************/

options nocenter;
options mprint mlogic symbolgen;


/***********************************************************************
  SECTION : ** print the datasets we will use in these examples;
************************************************************************/
proc print data=sashelp.class;
title "This is the data set used by most examples";
run;

proc print data=sashelp.shoes;
title "this dat set can be used by readers to create their own reports/macros";
run;
title "";

/***********************************************************************
  SECTION : **Global macros - to be avoided ***** Globals last too long and can interfere with other macros;
************************************************************************/
%let sex=F; /* global -- considered poor programming practice*/
proc print data=sashelp.class;
title "use the global macro to filter the data";
where sex=&sex;
run;

%let age=15;
proc print data=sashelp.class;
where sex="&sex" and age GE &age;
run;

%let WhereCls=where age= 14;
proc print data=sashelp.class;
title "the whole where clause can go in the macro - watch the ending semicolon - this is a good trick";
&WhereCls ;
run;
title "";

/***********************************************************************
  SECTION : Named parameters ARE LOCAL and good programming practice
            Also, this issustrates a trick to "get rid of the where clause"
            This shows how global macros can interfere with other macros
************************************************************************/
%macro Positional(wherecls=);
%put _user_;
proc print data=sashelp.class;
title "A local and global macro have the same name";
&WhereCls  ;
run;
%mend Positional;


%Positional(wherecls=);
%put _user_;


%Positional(wherecls=Where sex="M");
*callig this macro CHANGES THE VALUE OF THE GLOBAL MACRO VARIABLE!!!;
%put _user_;

/***********************************************************************
  SECTION :     Example of Good programming practice - make your code easy to read
                put comments in your macro definition and macro call
                You can have several positional parameters and comment them to make it easy to read
************************************************************************/
%macro Several(wherecls=Where age GT 12 /*Where age GT 12 this is a where clause without a semicolon*/
               ,obbs=(obs=10)           /*(obs=10) This limits the number ob obs - blank is allowed  */
                                );
/*This macro prints a few observations from the dataset SAShelp.class */
%put _user_;
proc print data=sashelp.class&obbs;
title "A local and global macro have the same name";
&WhereCls  ;
run;
%mend Several;

%Several(wherecls=                  /*this is a where caluse without a semicolon*/
         ,obbs=(obs=5)              /*This limits the number ob obs - blank is allowed  */
         );

%Several(wherecls=where sex="M"     /*this is a where caluse without a semicolon*/
         ,obbs=(obs=7)              /*This limits the number ob obs - blank is allowed  */
         );

/***********************************************************************
  SECTION : AUTOMATION TRICK: Runnning a report for values in a data set
                              Learn this Trick!  This is the current way to automate
                              This creates a global macro variable - not a good idea, but very commmon
************************************************************************/
proc SQL;
select distinct age into :agelist separated by " "
from sashelp.class;
quit;
%put _user_;

%macro listloop;
/*This macro loops over the ages in SAShelp.class and uses a where clause to produce a report for each age */
%local  loopcntr thisage;
%let loopcntr=1;
%do %while(%scan(&agelist,&loopcntr) NE );
  %let thisage=%scan(&agelist,&loopcntr);
  proc print data=sashelp.class;
  where age=&thisage;
  run;
  %let loopcntr=%eval(&loopcntr+1);
  %end;
%mend listloop;
%listloop;

***********************************************************************;
** if you put the sql inside the macro, the list variable becomes local;
    Same as above, but the macr variable is local
***********************************************************************;
%macro listloop2(varble=
                 ,CorN =c     /* is the variable char or numeric*/);
                 );
/*This macro does the same as the macro above, but does not use global macro variables*/
%local  loopcntr vallist thisval;

proc SQL;
select distinct &varble into :vallist separated by " "
from sashelp.class;
quit;
%put _user_;

%let loopcntr=1;
%do %while(%scan(&vallist,&loopcntr) NE );
  %let thisval=%scan(&vallist,&loopcntr);
  proc print data=sashelp.class;
  %if %upcase(&CorN)=C %then
        %do;
        where &varble="&thisval";
        %end;
  %else %if %upcase(&CorN)=N %then
        %do;
        where &varble=&thisval;
        %end;
  %else %do;
                put "ERROR IN MACRO CALL - check parameters"
                %end;
  run;
  %let loopcntr=%eval(&loopcntr+1);
  %end;
%mend listloop2;
%listloop2(varble=sex
                ,CorN =c     /* is the variable char or numeric*/);
                                );
%listloop2(varble=age
                ,CorN =N     /* is the variable char or numeric*/);
                                );

******************************************************************;
** An old way to do automate a process is to use a datastep and symput and && ;
   Use a looping macro with %do i= .....  and %end
*****************************************************************;
proc freq data=sashelp.class;
table age /out=valuelist(drop=count percent);
run;
proc print data=valuelist;
run;

data _null_;
set valuelist end=eof;
call symput("age"||left(put(_n_,3.0)),age); /*concatinate the ages  */
if eof=1 then
        do;
    call symput("upper",_n_);
        end;
run;
%put _user_;

%macro oldloop;
/*This macro uses a loop, and several macro variables, to produce the desired output */
%local i;
%do i= 1 %to &upper;
proc print data=sashelp.class;
title "i=&i and age=&&age&i";
where age=&&age&1;
run;
%end;
%mend oldloop;
%oldloop;

/***********************************************************************
  SECTION : using a control file to select macros and pass them parameters
                        using logic to create the control file.
            This is a powerful and advanced technique.  You meed to need automation to make this useful.
************************************************************************/
data contrlFile;
infile datalines missover firstobs=4;
input @1 Rtype $  @10 DSN $ @20 param1 $ @30 param2 $ ;
datalines;
         1         2         3         4         5         6         7         8
12345678901234567890123456789012345678901234567890123456789012345678901234567890
rtype    DSN       param1     Param2
show     class     age        15
show     class     sex        "M"
plot     class     age        height
run;
proc print data=contrlFile;
run;

**you use the variable Rtype to select
        a macro from your suite of pre-compiled macros--so lets write/compile some simple macros;
%macro listing(dsn=
               ,Numvar=
               ,val=
               );
proc print data=sashelp.&dsn;
title 'print where &numvar=&val;';
where &numvar=&val;
run;
title "";
%mend listing;

%macro plotting(dsn=
               ,Yvar=
               ,Xvar=
               );
proc plot data=sashelp.&dsn;
title 'plot &Yvar*&Xvar;';
plot &Yvar*&Xvar;
run;
title "";
%mend plotting;

**use call execute , and the control file, to "assemble" macro calls of the two macros above;
data _null_;
set contrlFile;
if     rtype="show" then
call execute('%listing(dsn='||dsn||',numvar='||param1||',val='||param2||')');
else if rtype="plot" then
call execute('%plotting(dsn='||dsn||',Yvar='||param1||',Xvar='||param2||')');
run;

/***********************************************************************
  SECTION : using a macro to save typing when renaming lots of variables;
            this macro is used/called as a data set option
            This can be really annoying to read.  Makes for a pretty complex data statement;
************************************************************************/
%macro rename(varcore=
             ,prefix=
             ,UpLim=
             );
%local i;
rename=(
%do i = 1 %to &uplim;
   &varcore&i=&prefix&varcore&i
%end;
)
%mend rename;

options mprint mfile nomlogic;
data testing (%rename(varcore=NRX,prefix=ave,Uplim=24));
run;


/***********************************************************************
  SECTION : Denormalizing data  
          or getting several totals in one pass through the data set
                  lets get the sum and average heights, ages and weights for males and females
          to make this more like a business problem we'll rename the variables
                  prod1 prod2 prod3 - we often use this logic to create vars like Qtr1 Qtr2 Qtr3 Qtr4
************************************************************************/
data raw;
infile datalines missover firstobs=5;
/*file of pharmacist sales by payor and merchansdise class*/
/*This will get the Q1 total by payor class */
/*There are other variables in the dataset so you can play with/expand the proglem*/
input @1 pharm $char3.  @10 pay_type $char1.
          @15 prod_type $   @30 q1  @35 Q2  @40 Q3 @45 Q4;
datalines;
         1         2         3         4         5         6
1234567890123456789012345678901234567890123456789012345678901234567890
c=cash  3=3rd party  h=HMO    D=Drug  E=equipment
Pharm payor  Product_type    q1   q2   q3   q4
bob      C    D              10   15   19   11
bob      3    E              23   12   24   11
bob      H    E              11   17   21   13
bob      C    D              13   16   23   15
bob      3    D              15   16   24   14
bob      H    D              10   17   25   17
Shu      C    D              13   11   27   19
Shu      3    E              16   12   29   17
Shu      H    E              17   17   20   14
Shu      C    D              12   14   21   12
Shu      3    D              14   17   22   14
Shu      H    D              10   16   21   14
;
run;

proc sort data=raw;
by pharm ;
run;

proc print data=raw;
run;

/*A zeroing macro and a summing macro*/
%macro Zro(prod= ,Zupper= );
   %do i=1 %to &Zupper;
        sum&prod&i=0 ;
   %end;
%mend zro;

%macro sumup(prod= , Supper=);
   %do i=1 %to &Supper;
        sum&prod&i+q&i;
   %end;
%mend sumup;

data one_line;
set raw;
by pharm;
if first.pharm then
        do;
     %zro(prod=CQ, zupper=4);%zro(prod=3Q, zupper=4);%zro(prod=HQ, zupper=4);
        end;

if       Pay_type="C" then
        do;
     %sumup(prod=CQ,supper=4);
        end;
Else if  Pay_type="3" then
        do;
     %sumup(prod=3Q,supper=4);
        end;
Else if  Pay_type="H" then
        do;
     %sumup(prod=HQ,supper=4);
        end;

if last.pharm then output;
run;

proc print data=one_line;
title "Data after collpsing to one line - this is the same technique as transposing";
run;
title "";


****capture the code to see what happened  *;
%macro What_gives;
options nosymbolgen;
data one_line;
set raw;
by pharm;
if first.pharm then
        do;
     %zro(prod=CQ, zupper=4);%zro(prod=3Q, zupper=4);%zro(prod=HQ, zupper=4);
        end;

if       Pay_type="C" then
        do;
     %sumup(prod=CQ,supper=4);
        end;
Else if  Pay_type="3" then
        do;
     %sumup(prod=3Q,supper=4);
        end;
Else if  Pay_type="H" then
        do;
     %sumup(prod=HQ,supper=4);
        end;
if last.pharm then output;
run;

proc print data=one_line;
title "Data after collpsing to one line - this is the same technique as transposing";
run;
title "";
%mend What_gives;



options mprint mfile;
filename mprint 'c:\test\capture_What_gives.txt';
 %what_gives;

options nomfile;




































***********************************************************************;
***********************************************************************;
***********************************************************************;
***********************************************************************;
/**********************************************************************
Macro code to accompany PPT slides:  Macro_5_2005_W_Ian_examples.ppt
***********************************************************************/
/***********************************************************************
  SECTION 0: General stuff - How to Capture macro code in a file
************************************************************************/
%let dset=one;
%let low=6;
%let high=12;

**First (and bad) example. this is NOT what you should do if you want to capture macro code-
the macro does not contain all the code we want to capture & so the capture file is incomplete*;

*run from line 15 to line 28 and see what you capture in capture_code.txt;
options mprint mfile;
filename mprint 'c:\temp\capture_code.txt';

data &dset;/* we will define and call the macro within the data step. */
/* routing output to a file starts and ends with the macro - we see only macro results*/
lookat=
%macro series; /* start of the macro*/
%do i= &low %to &high;
&i +
%end;
0;
%mend series;  /* end of the macro*/
%series;       /*Calling the macro - inside the data step*/
run;
options nomfile;

***** second example. this is what you SHOULD DO to capture macro code******;
***** Put a large block of code inside the macro - so you can see the macro results and the environment*;
options mprint mfile;
filename mprint 'c:\temp\capture_code2.txt';

%macro series; /* start of the macro is where we start capturing code*/
data &dset; /*we get this Data set name from a global macro variable*/
lookat=
%do i= &low %to &high;
&i +
%end;
0;
run;

proc print data=&dset;
run;

proc means data=&dset;
run;
%mend series; /* end of the macro is where we stop capturing code*/

%series;
options nomfile;

/***********************************************************************
  SECTION 1: Macros recall values for you
            %let in open code Creates a global Macro Variable
            first use
            macros are characters- always
            options
************************************************************************/
proc print data=sashelp.class;
run;
****;
%let MVsex="M";
**** you can look at macro values in many ways;
%put _user_;
%put &mvsex;
****lots of other stuff in the macro symbol table;
%put _automatic_;

proc print data=sashelp.class;
where sex="&MVsex";  /*we must fix the macro - or the where*/
run;

**options for checking - only one makes sense with this simple macro use;
options mprint mlogic nosymbolgen /*nomprint nomlogic nosymbolgen  */;
proc print data=sashelp.class;
where sex=&MVsex;  /*we must fix the macro - or the where*/
run;


%let month=january;
%let timep=&month;
%put _user_;

%let StNames =holding1;
%let holding1=holding2;
%let holding2=NY,NJ,DE;

%PUT &StNames;
%PUT &&&&&&&StNames;

%let ooops=%str(proc print; run;);

/***********************************************************************
  SECTION 2:macros conditionally recall values
                        The while-scan loop is becoming more common than a do loop
            LOOK AT THE LOG
                        First loop
            options mprint mlogic symbolgen
            local macro variable
            persistance of macro variables
************************************************************************/
%let MVSexlist= M F;
%put _user_;

%macro Prnt_sex;
options nocenter;

%macro skip;
%local MVSexList;
Proc sql;
title "Dynamically creating a macro variable";
title "";
select distinct sex into :MVSexList separated by " "
 from sashelp.class;
 quit;
%put _user_;
%mend skip;

%let sex_no=1;
%put _user_;
%do %while(%scan(&MVSexlist,&sex_no) NE ) ; /* do while this condition is true*/
        %let thissex=%scan(&MVSexlist,&sex_no);
        proc print data=sashelp.class;
        title "obs from sashelp.class for sex=&thissex";
        where sex="&thissex";
    run;
   %let sex_no=%eval(&sex_no+1); /*endless loop if this is forgotten*/
%end; /*end of the loop*/
%mend Prnt_sex;

%Prnt_sex;
***options;
options nosymbolgen nomlogic mprint; /*look at the nice clean log*/
%Prnt_sex;

/***********************************************************************
  SECTION 3:The top of the input stack
            Looking at the Macro Symbol table
      !!!!!!LOOKING AT THE SQL TABLE DICTIONARY.MACROS!!!!!!!
************************************************************************/
data year
     state;
infile datalines ;
input @1 year 4. @7 month $char3. @12 sales 3.0 @16 state $char2.;
datalines;
2000  jan  201 PA
2000  feb  202 PA
2000  mar  203 PA
2000  apr  204 PA
2000  may  205 PA
2000  jun  206 PA
2000  jul  207 PA
2000  aug  208 PA
2000  sep  209 PA
2000  oct  210 PA
2000  nov  211 PA
2000  dec  212 PA
2001  jan  211 DE
2001  feb  212 DE
2001  mar  213 DE
2001  apr  214 DE
2001  may  215 DE
2001  jun  216 DE
2001  jul  217 DE
2001  aug  218 DE
2001  sep  219 DE
2001  oct  220 DE
2001  nov  221 DE
2001  dec  222 DE
;
run;

proc print ;
run;
%let month=jan;
%put _user_;
%put _Global_;
%put _Automatic_;
%put _All_;

***start of reports***;
proc means data=year;
where month="&month";
run;

Proc print data=year;
where month="&month";
run;
****end of report*****;
*looking at the Macro Symbol table;
%put _user_;
%put _Global_;
%put _Automatic_;
%put _All_;

%put _user_;
*or;
proc sql;
describe table dictionary.macros;
select * from dictionary.macros
where scope="GLOBAL";
quit;

/***********************************************************************
  SECTION 4:Rules for puting stuff in the table
            The first semi-colon ends the command
            & and % are evaluated
************************************************************************/
** the first semocolon ends the command;
%let oops=proc print; run;
%put _user_;

%let oops2=%str(proc print; run;);
%put &oops2; /* note the not about unmasking*/

%put _user_; /* note the boxes*/

%put _all_;

**& and % (have to wait for a full understanding of %) are evaluated on macro assignment;
*put an & into the macro symbol table;
options mlogic symbolgen mprint;
%let time_period=&month;
%put _User_;

**show a % being evaluated as it goes into the symbol table***********************;
%let Max_of_bys=3;     /*count of "real" by macros - Do not count By0 */
%let By0=%str(             );
  /* Must have B0 to get the overall answers - fill with spaces to length of longest By*/
%let By1=%str(By spec);
%let By2=%str(By cat);
%let By3=%str(By segment);

/*concatinate the values of the by variabels into one macro var- use this for sorting*/
%macro makelist;
  %do cntr=1 %to &Max_of_bys; %substr(&&by&cntr,4) %end;
%mend makelist;

/*below we try to put a % into the Macro Symbol Table*/
options mprint mlogic symbolgen;
%let LstOfByVars=%makelist;
%put _user_; /*where does the one space between the variables come from?*/


/***********************************************************************
  SECTION 5: The &&&&&&
************************************************************************/
%let StNames =holding1;
%let holding1=holding2;
%let holding2=NY,NJ,DE;
%put _user_;

%put &&&StNames;

%put &&&&StNames;

**thanks to Shu Zan;
%put &&&&&&&StNames;

/***********************************************************************
  SECTION 6: Part 2 The macro Catalog- the storage place for Macro programs
                   Rules for Macro Compilation
                                   What gets compiled and what gets stored as text
                   The options Mlogic Mprint and Symbolgen
                   %str and %upcase
************************************************************************/
* a simple example - using Global Variables ;
%Macro Pprint;
title "Stuff for &State";
%if &sysday=Friday %then %do;
 proc freq data=sales;
 %str(;) /* an extra semicolon*/
 tables state/missing;  run;
%end;
proc print data=sales;
 where state="&state";
 sum sales;
 run;
%Mend Pprint;

%Macro Mmean;
 proc means data= sales;
 where %upcase(state)="&state";
 run;
%Mend Mmean;

%Macro Mmean;
proc means data= sales;
 where state="&state";
 run;
%Mend Mmean;


data sales;
infile datalines;
input state $  zip  $ sales Prod $ ;
datalines;
PA      19103   20 Gizmo
PA      19104   30 DoDad
PA      19104   20 DoDad
PA      19104   10 Gizmo
NJ      08103   20 Gizmo
NJ      08104   30 DoDad
NJ      08105   20 DoDad
NJ      08104   10 Gizmo
DE      18939   13 DoDad
DE      18922   12 Gizmo
DE      18877   17 Gizmo
DE      18877   7  Gizmo
DE      18877   27 DoDad
;

Options Mlogic Mprint Symbolgen;
%let state=PA; /*Global Variable  :-( */
%Pprint;

Options nomlogic nosymbolgen;
%let state=NJ; /*Global Variable  :-( */
%Mmean
%let state=DE; /*Global Variable  :-( */
%Pprint
%let state=PA; /*Global Variable  :-( */
%Mmean

*Good programming practice says do not use global variables******************;
** a better way to create macros is to pass the parameter so it is not global**;
%Macro Pprint2(filter_on=   /* this is a value of state.  It is unquoted e.g. DE*/
              );
%local Filter_on;
%let Filter_on=%upcase(&Filter_on);
%put _local_;
%put _user_;
title "A Freq report for &filter_on";
%if &sysday=Friday %then %do;
 proc freq data=sales;
 %str(;) /* an extra semicolon*/
 tables state/missing;  run;
%end;
proc print data=sales;
 where upcase(state)="&filter_on";
 sum sales;
 run;
%Mend Pprint2;

%Pprint2(filter_on=DE   /* this is a value of state it is unquoted e.g. DE*/
              );
%put _user_; /*note that the macro variable filter_on is GONE!!!!*/

****Example**this show the stuff goes to the top of the input stack*************************;
**Another Example: %statements just decide if code should be put at the opt of the input stack****************;
**imagine you are looping and each loop creates some result file (called LoopResult;
**in the first loop, the master file does not exist, so you create it;
**for other loops, you use a set statement to add the new file to the bottom of the master;

/*Lets be lazy and not write real looping code.  Just set the looping counter manually */

%let Nmbr_of_loop=1;
*%let Nmbr_of_loop=4;
%macro TopOTIStack;
   data MasterFile ; /*add sorting/tracking varaibles */
    set  %if &Nmbr_of_loop NE 1 %then
          %do; MasterFile  %end; LoopResult ;
         RUN;
%mend TopOTIStack;
options nomlogic mprint nosymbolgen;
%TopOTIStack
;

***Example**shows that stuff goes to the top of the input stack, even if the macro loops***;
**Another Example: %statements just decide if code should be put at the top of the input stack****************;
*Another example of looping putting code at the top of the input stack;
%macro TopOTIStack2(scanlist=11 14 16);
%let scan_count=1;
%do %while(%scan(&scanlist,&scan_count) NE ) ;
    %let this_age=%scan(&scanlist,&scan_count);
        options nocenter;
    proc print data=sashelp.class;
        title "class data for age=&this_age";
    where age=&this_age;
    run;
    %let scan_count=%eval(&scan_count+1);
%end;
%mend TopOTIStack2;
options nomlogic mprint nosymbolgen;
%TopOTIStack2;


**Example**Top of the Input Stack AND You don't need to execute a macro execute to test it ****;
**look a the log for these and see if it "looks right" - a visual test;
%Macro MakeYrList;
%do i=1960 %to 2001;      year&i%end;
%Mend MakeYrList;

option nomlogic nosymbolgen mprint;
data all_years;
set %MakeYrList;
;
run;

  ***re-write the macro clean up the log and make resulting code easier to see;
option nomlogic nosymbolgen mprint;
%Macro MakeYrList;
data all_years;
   set %do i=1960 %to 2001; year&i %end;
;
run;
%Mend MakeYrList;
%MakeYrList;

/***********************************************************************
  SECTION 7: Dump the macro catalog and see what is stored as text
************************************************************************/
*put a % into the macro symbol table;
%macro pprint(sex=);/*macro compile a macro */
 %let sex=%upcase(&sex);
 %if &sysday=FRIDAY
  %then %do;
   Proc print data=sashelp.class;
   where upcase(sex)="&sex";
   %str(str_&);
   %NRstr(Nrstr_&);
   %quote(Quote_&);
   %Bquote(BVQuote_&);
   %end;
run;
%mend pprint;

%macro listmac(libname=, macname=);/*do a hex dump of the Macro Catalog -look for text*/
  filename maccat catalog "&libname..sasmacr.&macname..macro";
  data _null_;
     infile maccat;
     input;
     list;
  run;
%mend;

%listmac(libname=work,macname=pprint);

%listmac(libname=work,macname=pprint);


/***********************************************************************
  SECTION 8:  The top of the input stack
                         Timing Issues in the evaluation of macro code
             The classic failure to interact with the data.
************************************************************************/
options nocenter;
proc print data=sashelp.air;
title "This is the raw data";
run;

proc summary      data=sashelp.air ;
output out= for_macro  (where=(_type_=0))   sum(air)=SmRv;
run;

proc print data=for_macro;
run;

data _null_;
set for_macro;
if SmRv GE 0  and SmRv LE 9999 then
        %let Tvar=LOW month; ;
else if SmRv GT 9999 then
        %let Tvar=HIGH month; ;
else    %let Tvar=missing; ;
run;

%put _user_;

proc print data=sashelp.air;
title "Sales Activity for this month was: &Tvar";
title2 "the title is ALWAYS WRONG";
run;

title "";

/***********************************************************************
  SECTION 9: part 3 done correctly - the problem is called a "timing issue"
            This is a common way to have the macro processor interact with data
            Loading a file into the Macro Symbol Table (an Array? without a number)
                        Using a data step (not the top of the input stack) to create macro variables
                        Creating a Macro Array
************************************************************************/
data Sales;
   input dept $ name $ sales;
   cards;
bedding  Watlee  18000
kitchen  Cannon  15000
tv       Jones   9000
carpet   Keller  20000
bedding  Ives    16000
bedding  Parker  9000
bedding  George  8000
;
run;

proc means data=sales;
 class dept;
   var sales;
   output out=stats sum=s_sal;
run;

proc print data=stats;
   var dept s_sal;
   title "Summary of Salary Information";
   title2 "For Dusty Department Store";
run;

data _null_;
   set stats;
if _n_=1 then
 call symput('s_tot',s_sal);
 else call symput('s'||dept,s_sal);
run;

/*above (creating a macro array  is NOT useful
  without seeing the technique for USING a macro array*/
/*Some code useing this array will be shown below Ex 11- think of the two examples together*/
%put _user_;

/***********************************************************************
  SECTION 10: Symget - a simple example of getting values from the macro table
                        Timing Issues
                        Using a data step (not the top of the Input stack) to get values
                                from the Macro Symbol Table
************************************************************************/
%let a1=the first;
%let a2=the middle;
%let a3=the end;

data new;
infile datalines;
input FRMPDV $ @@;
datalines;
A1 A2 A3  A2
run;

data new;
length dest $12;
set new ;
dest=symget(FRMPDV);
run;

proc print data=new;
run;

**put on the debugger;
data new/debug;
length dest $12;
set new ;
Executable="this line";
dest=symget(FRMPDV);
run;

/***********************************************************************
  SECTION 11: A more practical Example of Symget - use array from example 9
************************************************************************/
data new;
set sales;
pct_d= (sales /  symget('s'||dept)) *100;
stop1="here";

pct_t= (sales  /  &s_tot)  *100;
stop2="there";
run;

proc print data=new;
title "Sectoin 11 - using the array created in Secion 9";
run;

/***********************************************************************
  SECTION 12: A simple call execute - it put stuff on the input stack

************************************************************************/
*Call execute puts stuff on the input stack;
data _null_; /*this is pretty silly, but that is what execute does*/
call execute("Proc Print  data=SASHELP.class;");
call execute('title "New Title";');
call execute("run;");
run;

/***********************************************************************
  SECTION 13: A more practical call execute -
            Using Call Execute and a control file to automate a reporting process
                        USing a control file and call execute to select macros and pass them parameters
************************************************************************/
**the cool thing about what call execute does, is WHAT you can have put on the input stack..like macro calls;
**THE RAW DATA FILE - we want reports on this information*;
data NE_REGN;
infile datalines;
input state $  zip  $  PERSON $ sales Prod $ commission  ;
datalines;
PA      19103 Sue    20 Gizmo  1
PA      19104 Al     30 DoDad  2
PA      19104 Glenn  20 DoDad  2
PA      19104 Carlos 10 Gizmo  1
NJ      08103 Ramu   20 Gizmo  2
NJ      08104 Jim    30 DoDad  1
NJ      08105 Renzn  20 DoDad  3
NJ      08104 Donna  10 Gizmo  1
DE      18939 Eric   13 DoDad  2
DE      18922 Billy  12 Gizmo  1
DE      18877 Juan   17 Gizmo  2
DE      18877 Fan     7 Gizmo  1
DE      18877 Mike   27 DoDad  3
;
RUN;
proc print data=NE_REGN;
run;

**THE CONTROL FILE - MACRO NAMES AND PARMETERS - this is our Job schedule file!!!***;
data interest;
infile datalines missover;
input @1 state $ @5  rtype $;
datalines;
PA  Sales
NJ  Com
;
run;
proc print data=interest;
run;

**THE MACROS THAT ARE CALLED-this code defines,to SAS, the reports the customers want***;
%macro SSales(state);
proc print  data=NE_regn;
title "Sales Report: Sales for &state";
var person   Sales;
where state ="&state";
run;
%Mend SSales;

%macro CCom(state);
proc print  data=NE_regn;
title "Commission Report: Comm. for &state";
var person   commission;
*var person   comm; /*Try making this the var statement and look at the error*/
where state ="&state";
run;
%Mend CComm;

*the data null below runs the scheduling file - it calls macros and passes them parameters;
**turn on the macro tracking options and run;
*Call execute puts stuff on the input stack;

options mlogic mprint symbolgen;
data _null_;
set interest;
if      upcase(Rtype)="SALES"   then  call execute('%SSales('||state||')');
else if upcase(Rtype)="COM"     then  call execute('%CCom('||state||')');
run;

***********************************************************************;
***********************************************************************;
***********************************************************************;
***********************************************************************;
/***********************************************************************
  SECTION : Tramsposing data the J.B. Method
************************************************************************/
options nocenter;
/*****Raw data ************************;
/*Raw data - each subj has three lines that we want to collapse into one*/
data One_var(sortedby=name);
infile datalines missover firstobs=4;
input @1 name $ @10 test $ @20 value;
datalines;
         1         2         3         4         5
12345678901234567890123456789012345678901234567890
each subj has three lines that we want to collapse into one
Art      Weight    160
Art      Height     75
Art      Shoe_S   12.5
Bob      Weight    150
Bob      Height     67
Bob      Shoe_S    9.5
Russ     Weight    180
Russ     Height     68
Russ     Shoe_S   10.5
;
run;
proc print data=one_var;
run;

****Do the first transpose - one variable/no macros to see how it is done*******************;
data one_tr;
*each subj has three lines that we want to collapse into one;
retain wgt hgt ssz;
set One_var;
by name;
if first.name=1 then
   do;
   wgt  =0;
   hgt  =0;
   ssz  =0;
   end;
if      test="Weight" then Wgt=value;
Else if test="Height" then hgt=value;
Else if test="Shoe_S" then ssz=value;
if last.name=1 then
   do;
   output;
   end;
run;
proc print data=one_tr;
run;

*******do the second transpose - mumtiple variables with macros******************;
**raw data below********************;
data Mult_Vars;
infile datalines missover firstobs=4;
input @1 name $ @10 prod $ @20 Q1 @26 Q2 @31 q3 @36 q4;
datalines;
         1         2         3         4         5
12345678901234567890123456789012345678901234567890
name     product    Q1    Q2   Q3   Q4
Art      aspirin    60    55   50   45
Art      Ben_gay   175   170  165  160
Bob      aspirin    80    75   70   65
Bob      Ben_gay   135   130  125  120
Russ     aspirin    50    45   40   35
Russ     Ben_gay   120   115  120  115
;
run;
proc print data=Mult_Vars;
run;

***create the macros and Do the tranpose - data is NOT summarized****************;
%macro zero(Prod,max); /*this zeros variables */
    %do i=1 %to &max;
      &&prod._&i = 0;
        %end;
%mend zero;

%macro Transp(Prod,max); *this does the transposing - call this once for each product;
    %do i=1 %to &max;
      &&prod._&i = Q&i;
        %end;
%mend Transp;

*call the macros to do the transposing;
options mprint NOmlogic NOsymbolgen;
data wide;
set Mult_Vars;
if _N_=1 then
        do;
     %zero(asp,4);
         %zero(ben,4);
        end;
if     prod="aspirin" then
       do;
       %Transp(asp,4);
           end;
Else if prod="Ben_gay" then
       do;
       %Transp(ben,4);
           end;
run;
proc print data=wide;
run;

**A third transpose - put tow lines onto one - no summarizing done here****************;
%macro zero(Prod,max); /*use a loop to create the retain statements */
    %do i=1 %to &max;
          retain &&prod._&i;
      &&prod._&i = 0;
        %end;
%mend zero;

%macro Transp(Prod,max);
    %do i=1 %to &max;
      &&prod._&i = Q&i;
        %end;
%mend Transp;

options mprint NOmlogic NOsymbolgen;
data wide;
set Mult_Vars;
by name;
if _N_=1 then
        do;
     %zero(asp,4);
         %zero(ben,4);
        end;
if     prod="aspirin" then
       do;
       %Transp(asp,4);
           end;
Else if prod="Ben_gay" then
       do;
       %Transp(ben,4);
           end;
if last.name=1 then
       do;
       output;
           %zero(asp,4);
           %zero(ben,4);
           end;
run;
proc print data=wide;
run;



**Example 4 ** get patient total, hospital total and market total;
data Very_Mult;
infile datalines missover firstobs=4;
input @1 name $ @10 prod $ @19 cust $  @23 Q1 @29 Q2 @34 q3 @39 q4;
datalines;
         1         2         3         4         5
12345678901234567890123456789012345678901234567890
name     product       Q1    Q2   Q3   Q4
Art      aspirin  Pat  60    55   50   45
Art      aspirin  hsp  70    65   60   55
Bob      aspirin  Pat  80    75   70   65
Bob      aspirin  hsp  90    85   80   75
Russ     aspirin  Pat  60    55   50   45
Russ     aspirin  hsp  50    45   40   35
;
run;
proc print data=Very_Mult;
title "Multiple lines per salesman";
run;

%macro Czero(cust,max); /* zero the counting variables*/
    %do i=1 %to &max;
/*        retain &&prod._&i; */
      &&cust._&i = 0;
        %end;
%mend Czero;

%macro CTransp(cust,max); /*this is a summarization*/
    %do i=1 %to &max;
      &&cust._&i + Q&i; /* automatic retain*/
        %end;
%mend CTransp;

options mprint NOmlogic NOsymbolgen;
data Vry_wide;
set Very_Mult;
by name;
if _N_=1 then
        do;
     %Czero(hsp,4);
         %Czero(pat,4);
         %czero(mkt,4);
        end;

%CTransp(mkt,4);

if     cust="hsp" then
       do;
       %CTransp(hsp,4);
           end;
Else if cust="Pat" then
       do;
       %CTransp(pat,4);
           end;
if last.name=1 then
       do;
       output;
           %Czero(hsp,4);
           %Czero(pat,4);
           %Czero(mkt,4);
           end;
run;
proc print data=Vry_wide;
run;

****Example 5 capturing the macro code and different comments ***********************;
%macro make_file;
options mprint nomlogic nosymbolgen;
data Vry_wide;
set Very_Mult;
by name;
if _N_=1 then
        do;
         /*Comment type 1 The macro processor puts code on the top of the input stack */
     %Czero(hsp,4);
         %Czero(pat,4);
         %czero(mkt,4);
        end;
         *%comment type 2 The macro processor puts code on the top of the input stack ;
%CTransp(mkt,4);

if     cust="hsp" then
       do;
           %*comment type 3 The macro processor puts code on the top of the input stack;
       %CTransp(hsp,4);
           end;
Else if cust="Pat" then
       do;
           *% comment 4 The macro processor puts code on the top of the input stack;
       %CTransp(pat,4);
           end;
if last.name=1 then
       do;
       output;
         *% comment 5 The macro processor puts code on the top of the input stack;
           %Czero(hsp,4);
           %Czero(pat,4);
           %Czero(mkt,4);
           end;
run;

%mend make_file;


filename mprint "c:\temp\capture";
options mprint mfile;
%make_file;
options mprint nomfile;

***********************************************************************;
***********************************************************************;
***********************************************************************;
***********************************************************************;
/***********************************************************************
  SECTION MW1: Many ways to loop - this is a review of common macro automation techniques
************************************************************************/
%macro skip;
Understanding these macro calls is the goal of the macro cartoon
The business issue is: I am looking for datasets in SASHELP that I can use for examples
      in some new cartoons I am creating
These sections print 10 obs from ALL the data sets in SASHLEP that are of type=DATA;
%mend skip;

/*******************************************************
Program:        what SAS data
By:                     rml

Purpose:        in an automated manner, Print 10 obs form all the datafiels in SAShelp.
                        To investigate the files in SAShelp for use in teaching stuff
                        and to illustrate some of the different ways to automate a process
Infiles:

Outfiles:   xxxxxxxxxxxxxxxx
Started:        XXXXXXXXXXXXXXXXXX
Finished:   XXXXXXXXXXXXXXXXXX
Usage:      XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

Mod. Info:  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

*************************************************************************/
/***********************************************************
      SECTION MWL1: Look at the SAS dictionary tables - what is their structure
***********************************************************/
proc sql;
title "I can see vtable in the sas explorer, what is the name of the file in dictionary";
title2 "the common thinking is that sashelp tables run slower than dicionary tables";
describe view sashelp.vtable;
quit;

proc sql;
describe table dictionary.tables;
quit;

/*proc sql;*/
/*select memname into :file_list separated by " "*/
/*      from dictionary.tables*/
/*     where memtype="DATA" and libname="SASHELP";*/
/*quit; */
/*%put _user_;*/

/***********************************************************
      SECTION MWL2: Looping done the OLD way:
               using data _null_ and symput to create a macro array.
               Note the use of &&dsn_&i
***********************************************************/
data _null_;
set sashelp.vtable;
where memtype="DATA" and libname="SASHELP";
count+1;
call symput("dsn_"||left(put(count,3.0)),memname);
call symput("No_of_files",count); /*happens on every line- we just want the last one*/
                                  /*Often people use an "If end of file " to control this*/
run;
%put _user_;

%macro oldway;
%do i=1 %to &No_of_files;
 proc print data=sashelp.&&dsn_&i(obs=10);
 title "proc print data=sashelp.&&dsn_&i(obs=10);";
 run;
%end;
%mend oldway;

%oldway;

/***********************************************************
      SECTION MWL3: Newer way
               SQL puts multiple values into ONE macro and upper limit of loop = &SQLOBS
               use do loop to control the scanning
***********************************************************/
%macro Chk_files;
options nocenter;
proc sql noprint;
select memname into :file_list separated by " "
        from dictionary.tables
     where memtype="DATA" and libname="SASHELP";
quit;
%let SQLOBS=&SQLOBS; /*this is an automatic macro variable*/
%put _user_;

%do i=1 %to &SQLOBS /*also %while=(%scan(&file_list,&file_no) NE )) ;  */;
        %let thisfile=%scan(&file_list,&i);
        proc print data=sashelp.&thisfile(obs=10);
        title "10 obs from sashelp.&thisfile";
run;
%end; /*end of the loop*/
%mend Chk_files;

%Chk_files;


/***********************************************************
      SECTION MWL4:  Very Common Now
                SQL and scan the macro variable until scan returns a blank
***********************************************************/
%macro Chk_files;
options nocenter;
proc sql noprint;
select memname into :file_list separated by " "
        from dictionary.tables
     where memtype="DATA" and libname="SASHELP";
quit;
%put _user_;

%let file_no=1;
%do %while(%scan(&file_list,&file_no) NE ) ;
        %let thisfile=%scan(&file_list,&file_no);
        proc print data=sashelp.&thisfile(obs=10);
        title "10 obs from sashelp.&thisfile";
    run;
   %let file_no=%eval(&file_no+1); /*endless loop if this is forgotten*/
%end; /*end of the loop*/
%mend Chk_files;

%Chk_files;

/***********************************************************
      SECTION MWL5: Using the SQL and creating individual macro varaibles
               Using SQL to create a Macro Array
***********************************************************/

%macro Semioldway;
proc sql noprint;
select distinct memname into :infile1-:infile999
        from dictionary.tables
     where memtype="DATA" and libname="SASHELP";
quit;

%let Countoffiles=&SQLobs;
%Put _user_;

%do i=1 %to &Countoffiles;
 proc print data=sashelp.&&infile&i(obs=10);
 title "proc print data=sashelp.&&infile&i(obs=10);";
 run;
%end;
%mend Semioldway;

%Semioldway;
%put _user_;

/***********************************************************
      SECTION MWL6 : Using the SQL and creating individual macro varaibles
               Using SQL to create multiple Macro Arrays
               the trick is the : before the variable names
***********************************************************/
proc print data=sashelp.class;
run;

%macro MultArry;
proc sql noprint;
select distinct sex
               ,age
         into :sex1-:sex999
               ,:age1-:age999
        from sashelp.class
quit;

%let CountOfCombos=&SQLobs;
%Put _user_;

%do i=1 %to &CountOfCombos;
 proc print data=sashelp.Class;
 title "kids of Sex=&&sex&i and age =&&age&i";
 where sex="&&sex&i" and age=&&age&i;
 run;
%end;
%mend MultArry;

%MultArry;

/***********************************************************
      SECTION MWL7: Use SQL to create a control file - a list of macro parameters
                                and then use call execute to automate a job
      THE SQL MACRO TABLE
***********************************************************/
options nocenter;
proc sql noprint; /*This is a list of files -and will be a list of macro parameters */
create table file_list as
select memname
        from dictionary.tables
     where memtype="DATA" and libname="SASHELP";
quit;

%macro printMe(DSN=);/*dsn is a data set name,& we have a list of data sets in file_list*/
proc print data=sashelp.&dsn(obs=10);
title "10 obs using: proc print data=sashelp.&dsn(obs=10);";
run;
%mend PrintMe;

data _null_;
set file_list;
call execute('%PrintMe(DSN='||memname||')');
*call executes calls the macro and passes it parameters from the control file;
run;

/***********************************************************
   SECTION MWL8: In the old days, folks used SQL less often.  Here we use FREQ.
   Use FREQ, symput and looping to automate a report
   We want to print a report for each age (and don't want to use a by)
***********************************************************/
options nocenter;
proc print data=sashelp.class;
title "Look  at the data";
run;

********** Use Proc FreqGet a file of ages - ;
proc freq data=sashelp.class;
tables age/ out=ages list;
run;

proc print data=ages;
title "the age file";
run;
******SECTION: Create the macro vars - an array of macro variables;
data _null_;
set ages END=END;
call symput("age"||left(put(_n_,2.0)),left(put(age,2.0)));
   if END=1 then call symput("Numages",left(put(_n_,2.))); /*Only run once*/
run;

**A small digression here ******** look at the values of the macros ****;
%put _user_;
*or;
proc sql;
describe table dictionary.macros;
select * from dictionary.macros
where scope="GLOBAL";
quit;

%macro skip;
*This fails - the word dictionary is too long for a libname - must use SASHELP. in procs;
proc print data=dictionary.macros;
where scope="GLOBAL";
quit;
%mend skip;

*or;
proc print data=sashelp.Vmacro;
where scope="GLOBAL";
quit;

***  SECTION: use the macros in a loop;
%macro printing;
%do i=1 %to &Numages;
  options nocenter;
  proc print data=sashelp.class;
  title "the macro variable i has vlaue &i and age is &&age&1";
  where age= &&age&i;
  run;
%end;
%mend printing;

%printing;













***********************************************************************;
***********************************************************************;
***********************************************************************;
***********************************************************************;
***********************************************************************
  SECTION : Ian Whitlock's Complex Examples
***********************************************************************
/***********************************************************************
  SECTION IWCE 1: Details of complex looping
************************************************************************/

/******************************************
Program:                Macro_Loop_examples.sas
Programmed by:  Russ Lavery
Thanks to:              Ian Whitlock
Date:                   May 3, 2005
Purpose:                Produces reports (proc prints)
                                for each level of a variable
This is the code for a macro example for animation
****************************/
****"utility" macro definitions these take apart a two part SAS data set name*****;
%MACRO _libpart (data= ,debug=0);
/*MODULE        LIbpart
PURPOSE    Creates a macro variable holding library portion of a data set name
                this handles the user library
USAGE   %let libname=%Mlibpart(&data);
Details Searches &data for presence of a "."  when none found, creates a global variable libpart containing either "work" or "user" depending on the value of system option USER.  When "." is found, returns the libname portion of the data set name.
*/
%global libpart;
%IF %INDEX(&data,.)=0
 %THEN %DO;
         %IF %QSYSFUNC(GETOPTION(USER))=
             %THEN %let libpart=WORK;
         %ELSE %let libpart=USER;
       %END;
%ELSE %let libpart=%upcase(%SCAN(&data,1,.));
%if &debug NE 0 %then %put libpart=&libpart;
%MEND _libpart;
*%_libpart(data=sashelp.class)
;

%macro _mempart(data= ,debug=0 );
/*MODULE                _mempart
PURPOSE                 Creates a macro variable holding member portion of a SAS data set name
USAGE                   %let libname=%Mlibpart(&data);
Details                 Searches &data for presence of a "."  When none found,
                                loads "the original value; into a macro variable called mempart
                otherwise loads the portion following the period
*/
%global mempart;
%local debug;
%if %index(&data,.)=0
 %then %let mempart=&data;
 %Else %let mempart=%upcase(%scan(&data,2,.));
 %if &debug NE 0 %then %put mempart=&mempart;

%mend _mempart;
*%_mempart(data=sashelp.class)
*%put _user_;

/***
%let dsn=sashelp.class;
%let var=sex;
%let debug=1;
***/

option mlogic mprint symbolgen;

%macro Quick_rpt(DSN=sashelp.class  /* the data set to use*/
                ,Var=sex/*produce reports for each level of this var*/
                ,debug=0);
%let DSN=%upcase(&dsn); /*fix odd capitalization*/
%let var=%upcase(&Var);


*Leave system options as we saw them -I will change centering how is centering set;
PROC SQL NOPRINT;
   SELECT  setting into :tidy_up
   FROM DICTIONARY.OPTIONS
   /*FYI slower way: from sashelp.voption; remember that voption is a view*/
   where OPTNAME="CENTER" ;
   %put tidy_up has the value of  &tidy_up before the quit in the first SQL;
QUIT;
/*FYI Best way: forget the SQL, just use: %let tidy_up=sysfunc(getoptions(center));*/

*Change system options to what I want;
options nocenter;
*parse the dsn to create libpart and mempart macro vars;
%put _user_;
*options mlogic mprint symbolgen;
%_libpart (data=&dsn, debug=&debug);
%_mempart (data=&dsn, debug=&debug);

*We create a file with the levels of the var and levels of the var;
*used for where clause and to control looping;
proc freq data=&dsn NOPRINT;
  tables &var /out=_M_DS_1(drop=count percent);
  run;

*options mlogic mprint symbolgen;
data _null_; /*figure if the var is numeric or char */
    /*using vcolumn is not a good technique vcolumn is a view */
set sashelp.vcolumn
         (where=( libname="&libpart" and upcase(memname)="&mempart"
          and upcase(name)="&var"));
   call symput("C_or_N",left(upcase(type)));
run;
%put _user_;

data _null_; *create macro vars for each level and looping control variable;
  set _M_DS_1 end=eof;
   %IF &C_or_N =NUM %THEN %DO;
    call symput("Lvl_"||Left(put(_N_,3.0)),LEFT(PUT(&var,2.)));
   %END;
   %ELSE call symput("Lvl_"||Left(put(_N_,3.0)),LEFT(&var));
   ;
    if eof=1 then call symput("LvlsOfVar",left(put(_N_,2.0)));
   run;

/*** or you can do the same as above in one sql command and a % let - see below
Proc sql;
select distinct &var into :LvL_1-:lvl999
  from &DSN
  group by &var;
quit;
%let LvlsOfVar=&sqlobs;
run;
%put _user_;
******/

/** in keeping with the compare and contrast theme - this works as well
Proc sql noprint;
select upcase(type) into :C_or_N
from dictionary.columns
where libname="&libpart" and upcase(memname)="&mempart"
          and upcase(name)="&var" ;
%put &C_or_N=%upcase(&C_or_N);
quit;
**/

%put before the loop;
%put _user_;

*loop to print for every level of the var of interest ;
%do i=1 %to &LvlsOfVar;
      *- where clause differes for N and C;
      *chars in wheres must be quoted- numerics must NOT be quoted;
      %if  &C_or_N=CHAR %then %do;
               %let QtNotQt="&&Lvl_&i";
      %end;
      %else %if  &C_or_N=NUM %then  %do;
               %let QtNotQt=%left(&&Lvl_&i);
       %end;

    %put above the proc print i=&i;
    proc print data=&dsn;
       where &var=&QtNotQt;
       %put inside the loop, before the run i= &i;
    run;
       %put inside the loop, After the run i= &i;
%end;
%put Just before Quick_rpt shuts down,i= &i;

options &tidy_up;

%mend Quick_rpt;

*options nomprint nomlogic nosymbolgen;
options  mprint  mlogic  symbolgen;

%Quick_rpt(DSN=%str(sashelp.class)      /* the data set to use*/
                ,Var=age       /*produce reports for each level of this var*/
                 );


%Quick_rpt(DSN=%str(sashelp.class)      /* the data set to use*/
              ,Var=sex       /*produce reports for each level of this var*/
                 );


/***********************************************************************
  SECTION IWCE 2: The step boundary issue and how it sometimes screws up macros
                                  system commands execute immediately
                                  Timing
************************************************************************/
%macro Printplus(data=);

  title1 "My BIG Print";
  Proc Print data=&data;
  format _all_;
      %NextStep()

%mend printplus;

%macro
nextstep();
   title1 "Analysis Vars";
   Proc means data=&data;
   run;
%mend nextstep;

%printplus(data=sashelp.class)
;
/***********************************************************************
  SECTION IWCE 3: symput timing problem - when does symput actually execute
************************************************************************/

data W;
do obs=1 to 10 ;
 if obs <=5 then
  call symput ("x" ,"5" ) ;
  else
  call symput ("x" ,"0" ) ;

         y=&x;
         output;
  end;
run;

/***********************************************************************
  SECTION IWCE 4: Unmasking and the command stack in the Macro Processor
************************************************************************/
%let temp=c:\temp;
/***************** these fail,
     but they seemed like a good idea at the time;-9

   filename  InTemp "&temp\ex3.txt";

   filename InTemp  '&temp\ex3.txt';

   filename InTemp %str(%')&temp\ex3.txt%str(%');
**********************/
filename
  InTemp %unquote(%str(%')&temp\ex3.txt%str(%'));
