/******************************************************************
Section __: Reduce the number of obs in the data set: the slide is crowded
*******************************************************************/
data MyClass;
set SAShelp.class;
if mod(_N_,7) in (0,1,2,4,6,7) ;
label Name="Student name";
run;

/******************************************************************
Section __:  Select
*******************************************************************/
Options nocenter;
Proc SQL;
title "you can have correlated or un-correlated, sub-queries in the select";
title2 "They must return one value";
select name
   , sex
   ,(select max(sex) from SAShelp.class) as UnCorrQuery
   ,(Select avg(age) from SASHelp.class as Inner 
		where inner.sex=outer.sex) as CorrQuery
 
from MyClass as Outer
;quit;

Options nocenter;
Proc SQL;
title "Queries can not return multiple value to the Select";
title2 "They must return one value";
select name
   , sex
   ,(select distinct(sex) from SAShelp.class) as UnCorrQuery
 
from MyClass as Outer
;quit;


/******************************************************************
Section __: Having has a fee options
*******************************************************************/
Proc SQL;
title "returning one value is OK";
Select * , avg(age) as AvAge from MyClass as M
group by sex
having M.age = (select min(age) from MyClass where sex="M");
quit;

Proc SQL;
title "returning more than one value to an equals is NOT OK";
Select * , avg(age) as AvAge from MyClass as M
group by sex
having M.sex = (select distinct(sex) from MyClass);
quit;


Proc SQL;
title "returning more than one value to an in is OK";
Select * , avg(age) as AvAge from MyClass as M
group by sex
having M.sex in (select distinct(sex) from MyClass);
quit;


Proc SQL;
title "returning more than one value to an in is OK";
Select * , avg(age) as AvAge from MyClass as M
group by sex
having M.age in (select max(age) from MyClass group by sex);
quit;

Proc SQL;
title "returning more than one value to an EXISTS is OK";
Select * , avg(age) as AvAge from MyClass as M
group by sex
having name in (select name from MyClass where sex="M");
quit;


/******************************************************************
Section __: Subqueries  Where
*******************************************************************/
proc SQl;
title "A where subquery, with an IN, can return multiple values";
select name, age, sex
from MyClass
where name in(select name from MyClass group by sex having age=max(age));
run;

proc SQl;
title "A where subquery, with an =, can NOT return multiple values";
select name, age, sex
from MyClass as outer
where age =(select age from MyClass as inner group by sex having outer.sex=inner.sex);
run;

proc SQl;
title "A where subquery, with an =, can NOT return multiple values";
select name, age, sex
from MyClass as outer
where exists (select age from MyClass as Inner where inner.sex=outer.sex);
run;

proc SQl;
title "A where subquery, with an =, can NOT return multiple values";
select name, age, sex
from MyClass as outer
where exists (select name, age from MyClass as Inner where inner.name=outer.name and inner.sex NE outer.sex);
run;

proc SQl;
title "A where subquery, with an =, can NOT return multiple values";
select name, age, sex
from MyClass as outer
where exists (select name, age from MyClass as Inner where inner.name=outer.name and inner.sex eq outer.sex);
run;

/******************************************************************
Section __: From with a subquery
*******************************************************************/
proc SQL;
select name, age, sex 
	from( select * from MyClass where sex="M")
	order by age desc, name;
	run;


/******************************************************************
Section __: examples from slides
*******************************************************************/
	Proc SQL NUMBER _method _tree;
  select O.name , O.age, O.sex  
	   , (select Max(age) from MyClass) as MA
       , O.age-(select avg(age)          
                from Myclass as I
               where O.sex=I.sex )       as Diff_from_Sex_avg_age
       ,(select avg(age) from MyClass as I
               where O.sex=I.sex )      as Sex_avg_age
    from MyClass as O
;
quit;
options ls=256;
	Proc SQL NUMBER;
	/*easier to read and faster cause of fewer subqueries*/
  select O.name , O.age, O.sex  
	   , (select Max(age) from MyClass) as MA
       ,(select avg(age) from MyClass as I
               where O.sex=I.sex )      as Sex_avg_age
       , O.age-calculated sex_avg_age       as Diff_from_Sex_avg_age
    from MyClass as O
;
quit;
