/******************************************************************
Section __: Reduce the number of obs in the data set: the slide is crowded
*******************************************************************/
data MyClass;
set SAShelp.class;
if mod(_N_,7) in (0,1,2,4,6,7) ;
run;

/*****************************************************************
Section __: Use SQL to create the combination "Headers" that must be coded
    Cut and paste from SAS Listing back into program below        
******************************************************************/

data Group;
infile datalines truncover firstobs=2;
input @1 Group $char24. ;
datalines;
1234567890123456789012345678901234567890123456789012 
G1- NO  Group Statement*/                                 
G2- Yes Group statement*/
;               
run;


data having;
infile datalines truncover firstobs=2;
input @1 Having $char66. ;
datalines;
12345678901234567890123456789012345678901234567890123456 
/*H1- NO having clause                                */ 
/*H2- having applied to Summary variable              */ 
/*H3- having applied to Group variable                */ 
/*H4- having applied to Group and Summary Variables   */ 
/*H5- Having applied to Detail Data                   */ 
/*H6- Having applied to Summary and Detail Data       */ 
/*H7- Having applied to Group and Detail Data         */ 
/*H8- Having applied to Summary, Group and Detail Data*/ 
;
run;



data Select;
infile datalines truncover firstobs=2;
input @1 select $char63. ;
datalines;
123456789012345678901234567890123456789012345678901234567 
/*S1-No variables in Select                            
/*S2-Select has: NO  Detail - No  Group - YES Summary  
/*S3-Select has: NO  Detail - YES Group - NO  Summary  
/*S4-Select has: NO  Detail - YES Group - YES Summary  
/*S5-Select has: YES Detail - No  Group - No  Summary  
/*S6-Select has: YES Detail - No  Group - YES Summary  
/*S7-Select has: YES Detail - YES Group - No  Summary  
/*S8-Select has: YES Detail - YES Group - YES Summary  
;
run;

data break;
infile datalines truncover firstobs=2;
input @1 break $char95. ;
datalines;
1234567890123456789012345678901234567890123456789012345678901234567890123456789012345 
/************************************************************************************************/ 
;
run;

options ps=300 nocenter ls=120;
options ps=30 nocenter ls=120;
/*Generate section headers to be used in program below*/
proc sql number;
select Break, select, group, having, break 
from select, having, group, break;
quit; 

**this query is just to create macro variables holding interesting characeristics;
*we will put these values into following queries- just so we do not have to remember numbers;
options ls=120 nocenter;
proc SQL _method _tree; 
select  left(put(sum(sex="F"),2.0) ) as FemCount
	   ,left(put(sum(sex="M"),2.0) ) as MaleCount
       ,avg(height) as AllAvgHeight
       ,avg(age)   as AllAvgAge
       ,sum(height*(sex="F"))/sum(sex="F") as FemAvgH
	   ,sum(height*(sex="M"))/sum(sex="M") as MaleAvgH  
       ,sum(Age*(sex="F"))/sum(sex="F")    as FemAvgAge
	   ,sum(Age*(sex="M"))/sum(sex="M")    as MaleAvgAge  
into   :FemCount             ,:MaleCount 
       ,:AllAvgHeight        ,:AllAvgAge 
       ,:FemAvgH             ,:MaleAvgH  
       ,:FemAvgAge           ,:MaleAvgAge
  from MyClass 
;
%put _user_;
title1 "AllAvgAge=&AllAvgAge  and  AllAvgHeight=&AllAvgHeight";
title2 "FemAvgAge=&FemAvgAge  and MaleAvgAge=&MaleAvgAge";
title3 "FemAvgH=&FemAvgH     and MaleAvgH=&MaleAvgH";
title4 "FemCount=&FemCount   and   MaleCount=&MaleCount";


%put _user_;
title1 "AllAvgAge=&AllAvgAge &  FemAvgAge=&FemAvgAge & MaleAvgAge=&MaleAvgAge" ; 
title2 " AllAvgHeight=&AllAvgHeight & FemAvgH=&FemAvgH   & MaleAvgH=&MaleAvgH";
title3 "FemCount=&FemCount   and   MaleCount=&MaleCount";


proc means data=myclass;
class sex;
run;

/******************************************************************
Section __:  Pre-examples
*******************************************************************/
Proc SQL;
Select Name,   sex,   avg(height)
 From MYClass
 Where age GT 11
 Group by Sex
 Having Substr(name,1,1)='J' and height GT avg(height);

 Proc SQL;
 Select avg(height) as AvgHt
 From MYClass
;

 Proc SQL;
 Select avg(height) as AvgHt
 From MYClass
 Group by Sex
;

/*  1  **********************************************************************************************/
/*S1-No variables in Select                                      G1- NO  Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
options NOCENTER ls=120 ps=30;
Proc SQL _method _tree;
FOOTNOTE "NO VARIABLES IN SELECT RETURNS A SYNTAX ERROR";
select 
from MyClass as C;
run; /*whole data set is the group, One line through the grouping path*/
/*left in to prove that this generates a syntax Error*/
/*Must have variables in the select, Syntax Error not logical error*/

/*  2 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
proc SQL _method _tree;
title1 "overall_avg_height=&overall_avg_height";
title2 "FemAvgH=&FemAvgH and MaleAvgH=&MaleAvgH";
title3 "FemCount=&FemCount and MaleCount=&MaleCount";
footnote1 "We can tell, from the totals and averages produced by the query";
footnote2 "when in the process filters were applied to observations.";
footnote3 "Average, from query, equals overall average (see title), so no filters applied.";
select avg(height) as AvHt, Min(weight) as MinWt, max (height) as MaxHt
from MyClass as C;
run; /*whole data set is the group, One line through the grouping path*/
footnote "";

/*  3 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G1- NO  Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
*can not have group in select and no grouping variable;

/*  4 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
*can not have group in select and no grouping variablle;

/*  5 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G1- NO  Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote "all detail lines pass through detail path";
select name 
from MyClass as C;
run; 
footnote "";

**ppt exmple 1;
proc SQL _method _tree;
footnote "all detail lines pass through detail path";
select name , sex, age
from MyClass as C;
run; 
footnote "";

/*  6 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote "All detail lines go through detail path and one overall average goes through the summary path.";
footnote2 "support for the two path concept is in the note below:";
footnote3 "NOTE: The query requires remerging summary statistics back with the original data.";
select name, avg(age) as AvAge 
from MyClass as C;
run; /*all detail lines through detail path 
       and one overall average goes through the summary path */
    /*support for the two path concept is in the note below
    NOTE: The query requires remerging summary statistics back 
    with the original data.*/
footnote "";

/*  7 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G1- NO  Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
*can not have group in select and no grouping variablle;

/*  8 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
*can not have group in select and no grouping variablle;

/*  9 **********************************************************************************************
/*S1-No variables in Select                                      G2- Yes Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
proc SQL _method _tree;
select  
from MyClass as C
group by sex;
run; /*Code left in to prove that this generates a syntax Error*/

proc SQL _method _tree;
select  
from MyClass as C
group by sex
having sex="M";
run; /*Code left in to prove that this generates a syntax Error*/

/* 10 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
*ppt example 2;
proc SQL _method _tree;
footnote1 "two lines, and two variables, go through the grouping path."; 
footnote2 "sex is dropped late in the process and not not printed";
select  avg(age) as AvAge 
from MyClass as C
group by sex;
run; 
Footnote1;

/* 11 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G2- Yes Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
*PPt Example 3;
proc SQL _method _tree;
footnote1 "Lists all the sex values, ordered by sex";
footnote2 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause ";
footnote3 "because neither the SELECT clause nor the optional HAVING clause of the associated ";
footnote4 "table-expression referenced a summary function. ";
select  sex  
from MyClass as C
group by sex;
run; 
footnote "";

/* 12 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "Two obs through the grouping path, none through detail path. ";
footnote2 "Support for idea that all data goes through groping path is that there is ";
footnote3 "No note in the log that mentions a re-merging w/data afterwards";
select  sex, avg(age) as AvgAge  
from MyClass as C
group by sex;
run; 
footnote "";

/* 13 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G2- Yes Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "Lists all Name values, ordered by sex - but does not print sex";
footnote2 "name & sex sent through detial path, file is sorted and sex is dropped";
footnote3 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause ";
footnote4 "because neither the SELECT clause nor the optional HAVING clause of the associated ";
footnote5 "table-expression referenced a summary function. ";
select  name  
from MyClass as C
group by sex;
run; 
footnote "";

/* 14 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "detail lines, plus grouping var (to allow merging), go through detail path";
footnote2 "two obs (and two vars- sex and AvgAge) go through grouping path.  ";
footnote3 "After detail and grouping paths merged, the grouping var is dropped";
select  name , avg(age) 
from MyClass as C
group by sex;
run; 
footnote "";

/* 15 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G2- Yes Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "Lists all Name values and sex, ordered by sex ";
footnote2 "name & sex sent through detial path, file is sorted ";
footnote3 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause ";
footnote4 "because neither the SELECT clause nor the optional HAVING clause of the associated ";
footnote5 "table-expression referenced a summary function. ";
select  name , sex 
from MyClass as C
group by sex;
run; 

/* 16 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H1- NO having clause                                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "detail lines,plus grouping var, go through detail path";
footnote2 "two obs (Sex and AvgAve) go through grouping path.";
footnote3 "The detial and grouping files are merged, and grouping var NOT dropped";
select  name , sex , avg(age) as AvgAge
from MyClass as C
group by sex;
run; 
footnote "";

/* 17 **********************************************************************************************
/*S1-No variables in Select                                      G1- NO  Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
Proc SQL;
Select from MyClass having sex="F";
quit;

/*No variables in select produces syntax error*/


/* 18 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "Obs passes having test- Group path returns ONE observation";
footnote2 "one obs goes through grouping and filtering applied late"; 
select  avg(height) as AvHgt
from MyClass as C
having AvHgt GT 62;
run; 

proc SQL _method _tree;
footnote1 "Obs Fails having test- Group path returns ONE observation";
footnote2 "one obs goes through grouping and filtering applied late"; 
select  avg(height) as AvHgt
from MyClass as C
having AvHgt GT 63; 
quit; 

/* 19 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G1- NO  Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
 /*With NO summary,a having can not be applied to the summary*/

/* 20 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
 /*Can not have group in select without a grouping var*/


/* 21 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G1- NO  Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
 /*With NO summary,a having can not be applied to the summary*/


/* 22 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "All rows selected when comparison is to 62";
footnote2 "Sex for all obs sent through detail path, one obs sent through grouping";
footnote3 "overall average merged and the having is applied late in the process";
select sex, avg(height) as AvHgt
from MyClass as C
having AvHgt GT 62;/*True*/
run; 

proc SQL _method _tree;
footnote1 "NO rows selected when comparison is to 63";
footnote2 "Sex for all obs sent through detail path, one obs sent through grouping";
footnote3 "overall average merged and the having is applied late in the process";
footnote4 "when having is applied it filters out all rows";
select sex, avg(height) as AvHgt
from MyClass as C
having AvHgt GT 63; /*False*/
quit; 
footnote "";


/* 23 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G1- NO  Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
 /*With NO grouping statement ,can not have grouping var in select*/


/* 24 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
/*With NO grouping statement ,can not have grouping var in select*/

/* 25 **********************************************************************************************
/*S1-No variables in Select                                      G2- Yes Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
/*syntax error- must have some variable(s) in select clause*/

/* 26 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "Both obs return True for AvHgt GT 60";
footnote2 "Two obs (TWO Variables) sent through grouping- having applied to the obs";
footnote3 "Sex is dropped late in the process";
select avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 60;
run; 

proc SQL _method _tree;
footnote1 "One obs return True for AvHgt GT 63";
footnote2 "Two obs (One Variable) sent through grouping- having applied to the obs";
select avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 63;
run; 

proc SQL _method _tree;
footnote1 "NO obs return True for AvHgt GT 64";
footnote2 "Two obs (One Variable) sent through grouping- having applied to the obs";
footnote3 "Sex is dropped late in the process";
select avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 64; 
quit; 


/* 27 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G2- Yes Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/

/*Can not apply filter to summary if the select does not have a summary*/

/* 28 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "both obs return True for: having AvHgt GT 60;";
footnote2 "Two obs (Two Variables) sent through grouping- having applied to the obs";
select sex, avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 60;
run; 


proc SQL _method _tree;
footnote1 "ONE obs return True for: having AvHgt GT 63;";
footnote2 "Two obs (Two Variables) sent through grouping- having applied to the obs";
select sex, avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 63;/*One obs return True*/
run; /*Two obs (Two Variables) sent through grouping- having applied to the obs*/

proc SQL _method _tree;
footnote1 "NO obs return True for: having AvHgt GT 64;";
footnote2 "Two obs (Two Variables) sent through grouping- having applied to the obs";
select sex, avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 64; /*Zero obs return True*/
quit; /*Two obs sent through grouping- having applied to two the obs*/

/* 29 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G2- Yes Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
/*If no summary in select, can not apply a hving to summary*/


/* 30 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "code is having AvHgt GT 60; and 60 is lower than male or female avg";
footnote2 "name and sex sent through detail path,";
footnote3 "while sex and AvgAgt sent through grouping. ";
footnote4 "After merging, by sex, having applied to the obs & sex is dropped";
select name, avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 60; /*60 is lower than male or female avg*/
run; 


proc SQL _method _tree;
footnote1 "code is having AvHgt GT 63; and Female Avg <63 < Male Avg";
footnote2 "name and sex sent through detail path,";
footnote3 "while sex and AvgAgt sent through grouping. ";
footnote4 "After merging, by sex, having applied to the obs & sex is dropped";
select name, avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 63;
run; 

proc SQL _method _tree;
footnote1 "code is having AvHgt GT 64; Female Avg  < Male Avg <64";
footnote2 "name and sex sent through detail path,";
footnote3 "while sex and AvgAgt sent through grouping. ";
footnote4 "After merging, by sex, having applied to the obs & sex is dropped";
select name, avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 64; 
quit; 


/* 31 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G2- Yes Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
/*Can not apply having to summary, if there is no summary variable in select*/

/* 32 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H2- having applied to Summary variable              */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "Code is: having AvHgt GT 60; and 60 is lower than male or female avg";
footnote2 "name and sex sent through detail path,";
footnote3 "while sex and AvgAgt sent through grouping. ";
footnote4 "After merging, by sex, having applied to the obs";
select name, sex, avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 60; 
run; 

proc SQL _method _tree;
footnote1 "Code is: having AvHgt GT 63; Female Avg <63 < Male Avg";
footnote2 "name and sex sent through detail path,";
footnote3 "while sex and AvgAgt sent through grouping. ";
footnote4 "After merging, by sex, having applied to the obs";
select name, sex, avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 63;
run; 

proc SQL _method _tree;
footnote1 "Code is: having AvHgt GT 64; /*Female Avg  < Male Avg <63";
footnote2 "name and sex sent through detail path,";
footnote3 "while sex and AvgAgt sent through grouping. ";
footnote4 "After merging, by sex, having applied to the obs";
select name, sex, avg(height) as AvHgt
from MyClass as C
group by sex
having AvHgt GT 64; 
quit; 


/* 33 **********************************************************************************************
/*S1-No variables in Select                                      G1- NO  Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
/*Must have variables in Select, or SAS returns a syntax error*/


/* 34 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote "The average is key to understanding this. All Obs are used to calculate average & having is applied late. ";
footnote2 "The average shows all obs were used ot calculate average and the having was NOT turned into a where";
footnote3 "For each obs, sex sent through detail path,";
footnote4 "while AvgAgt (One var and One Obs) sent through grouping.";
footnote5 "After merging having applied to the obs";
footnote6 "tree shows that -LITC('M') was input to the aggregation";
select avg(height) 
from MyClass as C
having Sex ="M" and Name NE "Mike"; 
quit; 
 footnote "";


/* 35 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G1- NO  Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
/*can not be coded - what makes a variable a group in the select statement
   is that it is part of a group statement*/



/* 36 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
/*can not be coded - what makes a variable a group in the select statement
   is that it is part of a group statement*/



/* 37 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G1- NO  Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
/*can not be coded - what makes a variable a group in the Having statements
   is that it is part of a group statement*/



/* 38 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
/*can not be coded - what makes a variable a group in the Having statements
   is that it is part of a group statement*/


/* 39 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G1- NO  Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
/*can not be coded - what makes a variable a group in the Select and Having statements 
   is that it is part of a group statement*/


/* 40 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
/*can not be coded - what makes a variable a group in the Select and Having statements 
   is that it is part of a group statement*/


/* 41 **********************************************************************************************
/*S1-No variables in Select                                      G2- Yes Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
/*Must have variables in select or SAS returns a syntax error*/



/* 42 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote "The average is key to understanding this. Obs are used to calculate average for M & F & having is applied late. ";
footnote2 "Tree suggests having is applies at AGGR level, not data engine. The having was NOT turned into a where!";
footnote3 "";
footnote4 "One var and TWO Obs sent through grouping.";
footnote5 "having applied to the obs lae in the process";
footnote6 "tree shows that -LITC('M') was input to the aggregation";
select avg(Age) 
from MyClass as C
group by sex
having Sex ="M" ; 
quit; 
 footnote "";


/* 43 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G2- Yes Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "Lists all sex values filtered by the having, ordered by sex ";
footnote2 "sex sent through detail path, file is sorted and having filters the obs late";
footnote3 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause ";
footnote4 "because neither the SELECT clause nor the optional HAVING clause of the associated ";
footnote5 "table-expression referenced a summary function. ";
select Sex 
from MyClass as C
group by sex
having Sex ="M" ; 
quit; 
 footnote "";

/* 44 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "No obs pass down the Detail path";
footnote2 "On Group path, SAS creates two variables (sex and AvAge)";
footnote3 "The grouping file has two lines and the having filter happens late";
footnote4 "";
select Sex, avg(age) as AvAge 
from MyClass as C
group by sex
having Sex ="M" ; 
quit; 
 footnote "";

proc SQL _method _tree;
footnote1 "No obs pass down the Detail path";
footnote2 "On Group path, SAS creates three variables (sex, age and AvHgt)";
footnote3 "The group file has a row for each combo of Sex*Age ";
footnote4 "The having filter happens late";
select Sex, avg(Height) as AvHgt 
from MyClass as C
group by sex, age
having Sex ="M" ; 
quit; 
 footnote "";

/* 45 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G2- Yes Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "Output lists all the name values for males";
footnote2 "names pass down the detail path - no group path created- see note below";
footnote3 "Tree suggests that the filter is passed to the data engine";
footnote4 "  but sorting still occurs (by sex - pretty useless operation)";
footnote5 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause ";
footnote6 "because neither the SELECT clause nor the optional HAVING clause of the associated ";
footnote7 "table-expression referenced a summary function. ";
select name  
from MyClass as C
group by sex
having Sex ="M" ; 
quit; 
 footnote "";

proc SQL _method _tree;
footnote1 "Output lists all the name values for males";
footnote2 "names pass down the detail path - no group path created- see note below";
footnote3 "Tree suggests that the filter is passed to the data engine";
footnote4 "  but sorting still occurs (by sex and age)";
footnote5 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause ";
footnote6 "because neither the SELECT clause nor the optional HAVING clause of the associated ";
footnote7 "table-expression referenced a summary function. ";
select name  
from MyClass as C
group by sex, age
having Sex ="M" ; 
quit; 
 footnote "";

/* 46 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/

proc SQL _method _tree;
footnote1 "Output lists all the name values for males and avg age for males";
footnote2 "names & sex pass down the detail path ";
footnote3 "Group path creates a file (Sex and AvgAge) with 2 vars and 2 rows";
footnote4 "Summary stats are merged back in and having filter is applied late";
select name , avg(age)  AvgAge
from MyClass as C
group by sex
having Sex ="M" ; 
quit; 
 footnote "";

/* 47 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G2- Yes Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "Output lists all the name & sex values for obs were sex=M";
footnote2 "names and sex pass down the detail path - no group path created- see note below";
footnote3 "Tree suggests that the filter is passed to the data engine";
footnote4 "  but sorting still occurs (by sex and age)";
footnote5 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause ";
footnote6 "because neither the SELECT clause nor the optional HAVING clause of the associated ";
footnote7 "table-expression referenced a summary function. ";
select name , Sex 
from MyClass as C
group by sex
having Sex ="M" ; 
quit; 
 footnote "";

/* 48 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H3- having applied to Group variable                */
/**********************************************************************************************/

proc SQL _method _tree;
footnote1 "Output lists all the name & sex values for obs were sex=M & avg age for males";
footnote2 "names and sex pass down the detail path ";
footnote3 "Group path creates a file with 2 vars (sex & AvgAge) and 2 obs";
footnote4 "Tree suggests that the Having filter is applied late in the process";
footnote5 "Where sex="M" would be applied earlier";
select name , Sex , avg(age) as AvgAge
from MyClass as C
group by sex
having Sex ="M" ; 
quit; 
 footnote "";

/* 49 **********************************************************************************************
/*S1-No variables in Select                                      G1- NO  Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*Must have variables in select or SAS returns a syntax error*/

/* 50 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a group variable,we can not apply a having to the group variable*/


/* 51 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G1- NO  Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a group variable,we can not have a group in the select*/

/* 52 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a group variable,we can not have a group in the select*/


/* 53 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G1- NO  Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a group variable,we can not apply a having to the group varible*/


/* 54 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a group variable,we can not apply a having to the group varible*/


/* 55 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G1- NO  Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a group variable,we can not apply a having to the group varible*/


/* 56 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a group variable,we can not apply a having to the group varible*/


/* 57 **********************************************************************************************
/*S1-No variables in Select                                      G2- Yes Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a group variable,we can not apply a having to the group varible*/

/* 58 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "Detail path never activates";
footnote2 "Group path creates a file with 2 vars (sex and AvgAge) and 2 rows ";
footnote3 "Tree suggests that having filtering is applied late in the process";
footnote4 "";
select  avg(age) as AvgAge
from MyClass as C
group by sex
having Sex ="M" and AvgAge GT 13.3 ; 
quit; 
 footnote "";

proc SQL _method _tree;
footnote1 "Detail path never activates";
footnote2 "Group path creates a file with 2 vars (sex and AvgAge) and 2 rows ";
footnote3 "Tree suggests that having filtering is applied late in the process";
footnote4 "";
select  avg(age) as AvgAge
from MyClass as C
group by sex
having Sex ="F" and AvgAge GT 13.3 ; 
quit; 
 footnote "";


/* 59 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G2- Yes Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a summary variable in the select,
    we can not apply a having to the summary varibale*/


/* 60 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "No activity in the detail path";
footnote2 "The grouping path creates a 2 var (sex & AvgAge) by 2 obs data set";
footnote3 "All the girl obs were used ot calculate the average";
footnote4 "Tree suggest that the having filtering is applied late in the process";
select  sex, avg(age) as AvgAge
from MyClass as C
group by sex
having Sex ="F" and AvgAge GT 12 ; 
quit; 
 footnote "";

 proc SQL _method _tree;
footnote1 "No activity in the detail path";
footnote2 "The grouping path creates a 2 var (sex & AvgAge) by 2 obs data set";
footnote3 "All the girl obs were used to calculate the average";
footnote4 "Tree suggest that the having filtering is applied late in the process";
select  sex, avg(age) as AvgAge
from MyClass as C
group by sex
having Sex ="F" and AvgAge LT 12 ; 
quit; 
 footnote "";


/* 61 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G2- Yes Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a summary variable in the select,
    we can not apply a having to the summary variable*/


/* 62 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "A List of male/female names and sexes go on detail path";
footnote2 "The grouping path creates a 2 var (sex & AvgAge) by 2 obs data set";
footnote3 "The gruping and the detail files are merged by sex, then sex is dropped";
footnote4 "The having filter is applies late in the process";
Footnote5 "The average age is the average age for all boys";
select  name, avg(age) as AvgAge
from MyClass as C
group by sex
having Sex ="M" and AvgAge GT 12 ; 
quit; 
 footnote "";


/* 63 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G2- Yes Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
/*If we do not have a summary variable in the select,
    we can not apply a having to the summary variable*/


/* 64 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H4- having applied to Group and Summary Variables   */
/**********************************************************************************************/
proc SQL _method _tree;
footnote1 "A List of male/female names and sexes go on detail path";
footnote2 "The grouping path creates a 2 var (sex & AvgAge) by 2 obs data set";
footnote3 "The gruping and the detail files are merged by sex";
footnote4 "The having filter is applies late in the process";
Footnote5 "The average age is the average age for all boys";
select  name, sex, avg(age) as AvgAge
from MyClass as C
group by sex
having Sex ="M" and AvgAge GT 12 ; 
quit; 
 footnote "";

proc SQL _method _tree;
footnote1 "A List of male/female names and sexes go on detail path";
footnote2 "The grouping path creates a 2 var (sex & AvgAge) by 2 obs data set";
footnote3 "The gruping and the detail files are merged by sex";
footnote4 "The having filter is applies late in the process";
Footnote5 "The average age is the average age for all boys";
select  name, sex, avg(age) as AvgAge
from MyClass as C
group by sex
having Sex ="F" and AvgAge GT 13.5 ; 
quit; 
 footnote "";


/* 65 **********************************************************************************************
/*S1-No variables in Select                                      G1- NO  Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
/*we must have variables in the select, or SAS returns an error*/


/* 66 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "A List of all male/female names go on detail path";
footnote2 "The grouping path creates a 1 var (AvgAge) by 1 obs data set";
footnote3 "The gruping is done on the whole data set and merged with the detial file";
footnote4 "After merging the name varaible is dropped";
footnote5 "The having filter is applies late in the process";
Footnote6 "After merging the name varaible is dropped";
select /*name,*/ avg(age) as AvgAge
from MyClass as C
having substr(name,1,1)in ("J","M") ; 
quit; 
 footnote "";


/* 67 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G1- NO  Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
/*if we do not have a grouping statement, 
        we cna not have a grouping variable on the select*/


/* 68 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
/*if we do not have a grouping statement, 
        we cna not have a grouping variable on the select*/



/* 69 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G1- NO  Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "A List of names having substr(name,1,1)in ('J','M')  go on detail path";
footnote2 "The grouping path creates a 1 var (AvgAge) by 1 obs data set";
footnote3 "Tree suggests that the having was changed to a where";
footnote4 "and applied early - before the first occurrance of SRC";
select name
from MyClass as C
having substr(name,1,1)in ("J","M") ; 
quit; 
 footnote "";


/* 70 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "A List of ALL names having substr(name,1,1)in ('J','M')  goes on detail path";
footnote2 "The grouping path creates a 1 var (AvgAge) by 1 obs data set";
footnote3 "Average shown is average for whole data set- no filtering before calculation of avg";
footnote4 "The average was merged to all the names";
footnote5 "the having was applied late in the process";
select name , avg(age) as AvgAge
from MyClass as C
having substr(name,1,1)in ("J","M") ; 
quit; 
 footnote "";

/* 71 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G1- NO  Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
/*if we do not have a grouping statement, 
        we cna not have a grouping variable on the select*/


/* 72 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
/*if we do not have a grouping statement, 
        we can not have a grouping variable on the select*/


/* 73 **********************************************************************************************
/*S1-No variables in Select                                      G2- Yes Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
/*If we have no variables in the select statement, SAS returns a syntax error*/


/* 74 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "A List of ALL names & sex  goes on detail path";
footnote2 "The grouping path creates a 2 var (sex & AvgAge) by 2 obs data set";
footnote3 "Averages shown is average for all boys and all girls! NO filtering before calculation of avg";
footnote4 "The averages were merged to all the names by sex, and sex was dropped";
footnote5 "the having was applied late in the process, and the name variable was dropped";
select  avg(age) as AvgAge
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") ; 
quit; 
 footnote "";


/* 75 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G2- Yes Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "A List of ALL names & sex  goes on detail path";
footnote2 "Since ther are no summary stats on select, No obs go to the grouping path";
footnote3 "The having was applied late in the process, and the name variable was dropped";
footnote4 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause ";
footnote5 "because neither the SELECT clause nor the optional HAVING clause of the associated ";
footnote6 "table-expression referenced a summary function. ";
select  sex
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") ; 
quit; 
 footnote "";


/* 76 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "A List of ALL names & sex  goes on detail path";
footnote2 "The grouping path creates a 2 var (sex & AvgAge) by 2 obs data set";
footnote3 "Averages shown is average for all boys and all girls! NO filtering before calculation of avg";
footnote4 "The averages were merged to all the names by sex, and sex was retained";
footnote5 "The having was applied late in the process, and the name variable was dropped";
select  sex, Avg(age) as AvgAge
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") ; 
quit; 
 footnote "";


/* 77 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G2- Yes Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "If no summary variable in select, groups are changed to order";
footnote2 "Tree suggests having is changed to a where and applied early";
select  name, sex
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") ; 
quit; 
 footnote "";


/* 78 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "Averages are for all girls and all bys- no obs filter before average calculation";
footnote2 "All names & sex are sent to detail path";
footnote3 "Group path creates a 2 var (aex & avgAge) by 2 row data set";
footnote4 "Averages are merged in by sex";
footnote5 "Tree suggests that the having filter is applied late in the process";
select  name, avg(age) as AvgAge
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") ; 
quit; 
 footnote "";

/* 79 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G2- Yes Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "A List of FILTERED names & sex  goes on detail path";
footnote2 "Since there are no summary stats on select, No obs go to the grouping path";
footnote3 "since no summary function on select, group changed to Sort";
footnote4 "having changed to a where AND APPLIED EARLY"; 
footnote5 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause ";
footnote6 "because neither the SELECT clause nor the optional HAVING clause of the associated ";
footnote7 "table-expression referenced a summary function. ";
select  name, sex 
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") ; 
quit; 
 footnote "";

/* 80 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H5- Having applied to Detail Data                   */
/**********************************************************************************************/
/*Example*/
proc SQL _method _tree; 
footnote1 "A List of UN-FILTERED names & sex  goes on detail path";
footnote2 "Since there are no Where clauses, all obs flow into the grouping path";
footnote3 "After the groupng is done, the two paths merge- having all obs in detal path";
footnote4 "having changed applied after the merge - detail obd filtered"; 
footnote5 "the average shown in the listing is the average for all obs in RAW";
footnote6 "because neither the SELECT clause nor the optional HAVING clause of the associated ";
footnote7 "table-expression referenced a summary function. ";
select  name, sex, mean(age) 
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") ; 
quit; 
 footnote "";

/* 81 **********************************************************************************************
/*S1-No variables in Select                                      G1- NO  Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
/*If no variables in select, SAS returns a syntax Error*/

/* 82 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "The detal variable(name) in the having sends all names & sex down detail path";
footnote2 "Since there is a summary on select, all obs go into the grouping path";
footnote3 "since there is no grouping var, all vars go on one group";
footnote4 "Overall avg is merged to all obs and fname filtering is applied";
select  mean(age) as mean_age
from MyClass as C
having substr(name,1,1)in ("J","M") and mean_age GT 12; 
quit; 
 footnote "";

/* 83 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G1- NO  Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
/*If there is no group statement we cn not have a group variable on the select  */

/* 84 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
/*If there is no group statement we cn not have a group variable on the select  */

/* 85 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G1- NO  Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/

proc SQL _method _tree; 
footnote1 "witout a summary in the select, you can not apply having to summary";
footnote2 "SAS returns syntax error";
footnote3 "";
footnote4 "";
select  name 
from MyClass as C
having substr(name,1,1)in ("J","M") and mean_age GT 12; 
quit; 
 footnote "";

/* 86 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/

proc SQL _method _tree; 
footnote1 "detail(name) in select or in having sends names to detail";
footnote2 "since there is no where, all names go to detial path";
footnote3 "since there is no where, all obs go to group path";
footnote4 "since there is no group statement, there is one group";
footnote5 "after merging, the having is applied";
footnote6 "";
footnote7 "";

select  name, avg(age) as mean_age 
from MyClass as C
having substr(name,1,1)in ("J","M") and mean_age GT 14;
/*having substr(name,1,1)in ("J","M") and mean_age GT 14; */
quit; 
 footnote "";     

/* 87 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G1- NO  Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
/*without a group statement, we can not have a group var in the select*/

/* 88 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
/*without a group statement, we can not have a group var in the select*/

/* 89 **********************************************************************************************
/*S1-No variables in Select                                      G2- Yes Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
/*without variables in the select statement, SAS returns a syntax error*/

/* 90 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/

proc SQL _method _tree; 
footnote1 "detail(name) in select or in having sends names to detail";
footnote2 "since there is no where, all names go to detail path";
footnote3 "since there is no where, all obs go to group path";
footnote4 "since there is a group by sex statement, there are two groups (averages)";
footnote5 "after merging by sex, sex is dropped, and the having is applied";
footnote6 "Averages shown are the averages for thw all maes and females";
footnote7 "";

select  avg(age) as mean_age, sex 
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") and mean_age GT 13;
/*having substr(name,1,1)in ("J","M") and mean_age GT 14; */
quit; 
 footnote "";     

/* 91 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G2- Yes Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/

proc SQL _method _tree; 
footnote1 "SYNTAX ERROR ";
footnote2 "The having wants to see a variable named mean_age ";
footnote3 "The variable is not created in select, and having can not create a summary";
footnote4 "since there is no where, all obs go to group path";

select  sex 
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") and mean_age GT 13;
/*having substr(name,1,1)in ("J","M") and mean_age GT 14; */
quit; 
 footnote "";     

/* 92 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
/*with no group statement, we can not put a group var in the select*/

/* 93 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G2- Yes Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
/*with no summary variable in the select, 
   you can not filer on a summary statement in the having*/
/*the ahving will not create a summary variable, think of the SQL path*/

/* 94 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "Detail, in select, sends all obs through detail path";
footnote2 "summary in select sends all obs through grouping path, creating a 1 by 1 file";
footnote3 "the merge happens and then obs get filtered";
footnote4 "since there is no where, all obs go to group path";

select  name, avg(age) as mean_age
from MyClass as C
having substr(name,1,1)in ("J","M") and mean_age GT 13;
/*having substr(name,1,1)in ("J","M") and mean_age GT 14; */
quit; 
 footnote "";     

/* 95 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G2- Yes Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
/*No summary in select, so no obs sent to grouping 
   - having does not create summary file*/

proc SQL _method _tree; 
footnote1 "No summary in select, so no obs sent to grouping - having does not create summary file";
footnote2 "Syntax error";

select  name, sex 
from MyClass as C
having substr(name,1,1)in ("J","M") and mean_age GT 13;
/*having substr(name,1,1)in ("J","M") and mean_age GT 14; */
quit; 
 footnote "";     

/* 96 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H6- Having applied to Summary and Detail Data       */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "Detail in select sends all obs into setail path";
footnote2 "Avg in select sends all obs through the grouping path (1 by 1 file)";
footnote3 "the average shown is for all obs";
footnote4 "Details and summary are merged and having applies filtering";
footnote5 "";
footnote6 "";

select  name, sex , avg(age) as Avg_age
from MyClass as C
having substr(name,1,1)in ("J","M") and Avg_age GT 13;
/*having substr(name,1,1)in ("J","M") and avg_age GT 14; */
quit; 
 footnote "";     

/* 97 **********************************************************************************************
/*S1-No variables in Select                                      G1- NO  Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
/*If no variables in the select, SAs returns a syntax error*/

/* 98 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
/*with no group statement, we can not apply a having to the group statement*/

/* 99 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G1- NO  Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
/*with no group statement, we can not have a group in the statement*/

/*100 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
/*with no group statement, we do not have a group in the having*/

/*101 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G1- NO  Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
/*with no group statement, we do not have a group in the having*/

/*102 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
/*with no group statement, we do not have a group in the having*/

/*103 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G1- NO  Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
/*with no group statement, we do not have a group in the having*/

/*104 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
/*with no group statement, we do not have a group in the having*/

/*105 **********************************************************************************************
/*S1-No variables in Select                                      G2- Yes Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
/*with no variables in the select, SAS returns an error*/

/*106 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "No Detail var in select, but substring, in having, sends all obs through detail path";
footnote2 "summary in select sends sex for all obs through grouping path, creating a 2 (sex & AvgAge) var by 2 obs file";
footnote3 "the merge happens and then obs get filtered";
footnote4 "since there is no where, all obs go to group path";
footnote5 "NOTE: The query requires remerging summary statistics back with the original data.";
footnote5 "We get a line for each child with the right name AND whose gender's AvgAge  meets the having";
select  avg(age) as AvgAge
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") and AvgAge GT 13;
having substr(name,1,1)in ("J","M") and mean_age GT 14; 
quit; 
 footnote "";     

/*107 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G2- Yes Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "No Detail var in select, but substring, in having, sends all obs, and their sex, through detail path";
footnote2 "Since no summary var in select, no obs go to the summary path ";
footnote3 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause because neither the SELECT clause nor the";
footnote4 "optional HAVING clause of the associated table-expression referenced a summary function.";
footnote5 "Since no Summary function in select, group by turns into an order by";
select  sex 
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") and Sex="F";

quit; 
 footnote "";     



/*108 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "No Detail var in select, but substring, in having, sends names for all obs, and their sex, through detail path";
footnote2 "avg function sends all obs to the summary path ";
footnote3 "Avg age (by gender) is merged, by gender, onto all names ";
footnote4 "Having filtering is applied";
footnote5 "NOTE: The query requires remerging summary statistics back with the original data.";
select  sex, avg(age) as AvgaAge 
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") and Sex="F";

quit; 
 footnote "";  


/*109 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G2- Yes Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "Detail var in select sends all names,  having adds sex var to detail path";
footnote2 "Since no summary var in select, no obs go to the summary path ";
footnote3 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause because neither the SELECT clause nor the";
footnote4 "optional HAVING clause of the associated table-expression referenced a summary function.";
footnote5 "Since no Summary function in select, group by turns into an order by";
select  name  
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") and Sex="F";

quit; 
 footnote "";


/*110 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "Detail var in select sends all names,  group adds sex var to detail path";
footnote2 "AvgAge sends all obs to to the summary path ";
footnote3 "AvgAge merged with detail and having subsetting is applied";
select  name, avg(age) as AvgAge  
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") and Sex="F";

quit; 
 footnote "";


/*111 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G2- Yes Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "Tree suggests that the having was changed to a where and applied early";

footnote3 "Detail var (name) in select sends all names,  group adds sex var to detail path";
footnote4 "Since no summary function in select, no obs go to group path - group by changed to order by";
footnote5 "AvgAge merged with detail and having subsetting is applied";
footnote6 "WARNING: A GROUP BY clause has been transformed into an ORDER BY clause because neither the SELECT clause nor the";
footnote7 "optional HAVING clause of the associated table-expression referenced a summary function.";
footnote8 "Since no Summary function in select, group by turns into an order by";

select  name, Sex  
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") and Sex="F";

quit; 
 footnote "";

 
/*112 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H7- Having applied to Group and Detail Data         */
/**********************************************************************************************/
proc SQL _method _tree; 
footnote1 "Detail var (name) in select sends all names,  group adds sex var to detail path";
footnote4 "summary function in select, sends obs to group path and creates a 2 var (sex & AvgAge) by 2 row data set";

select  name, Sex , avg(age) as AvgAge  
from MyClass as C
group by sex
having substr(name,1,1)in ("J","M") and Sex="F";

quit; 
 footnote "";


/*113 **********************************************************************************************
/*S1-No variables in Select                                      G1- NO  Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/


/*114 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/




/*115 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G1- NO  Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/




/*116 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/



/*117 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G1- NO  Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/




/*118 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G1- NO  Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/



/*119 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G1- NO  Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/




/*120 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G1- NO  Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/




/*121 **********************************************************************************************
/*S1-No variables in Select                                      G2- Yes Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/




/*122 **********************************************************************************************
/*S2-Select has: NO  Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/


/*123 **********************************************************************************************
/*S3-Select has: NO  Detail - YES Group - NO  Summary            G2- Yes Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/


/*124 **********************************************************************************************
/*S4-Select has: NO  Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/




/*125 **********************************************************************************************
/*S5-Select has: YES Detail - No  Group - No  Summary            G2- Yes Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/




/*126 **********************************************************************************************
/*S6-Select has: YES Detail - No  Group - YES Summary            G2- Yes Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/





/*127 **********************************************************************************************
/*S7-Select has: YES Detail - YES Group - No  Summary            G2- Yes Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/




/*128 **********************************************************************************************
/*S8-Select has: YES Detail - YES Group - YES Summary            G2- Yes Group Statement      */
/*H8- Having applied to Summary, Group and Detail Data*/
/**********************************************************************************************/
	
